from utils import*
from blochwave import bloch2D as bl          ;imp.reload(bl)
from blochwave import bloch as bl0           ;imp.reload(bl0)# load_Bloch
from wallpp import config as cg      ;imp.reload(cg)
plt.close('all')

class bloch2D_Cont:
    def __init__(self,u,tag='test',path='dat',run=False,bargs={}):
        self.u=u
        self.path=path
        self.tag=tag
        if run:self._run(bargs)
        self._set_df_G()

    def _run(self,bargs):
        for i,u0 in enumerate(self.u):
            name = '%s_%s' %(self.tag,str(i).zfill(3))
            b0  = bl.Bloch2D(file=self.tag,u=u0,path=self.path,name=name,**bargs)
            hk = [str((h,k)) for h,k in zip(*b0.df_G[['h','k']].values.T)]
            b0.df_G.index = hk
            b0.save()

    def _set_df_G(self):
        b = self.load_b()

        hk = np.unique(np.hstack([b0.df_G.index.values for b0 in b]))
        df_G=pd.DataFrame(np.zeros((hk.size,3)),columns=['qx','qy','I'],index=hk)
        for i,b0 in enumerate(b):
            df_G.loc[b0.df_G.index,['qx','qy']] = b0.df_G[['qx','qy']].values
            df_G.loc[b0.df_G.index,'I'] += b0.df_G.I.values
        df_G.loc[str((0,0)),'I'] = 0
        self.df_G=df_G


    #####################################################################
    #### available
    #####################################################################
    def load_b(self):
        return [bl0.load_Bloch(self.path,
            '%s_%s' %(self.tag,str(i).zfill(3)),v=0) for i,u0 in enumerate(self.u)]

    def show_I(self,thick=None,fz=abs,ms=80,pOpt='im',caxis=[],
        **kwargs):
        if thick:self.set_I(thick)
        if isinstance(fz,str):
            if fz=='log':fz = lambda x:np.log10(np.maximum(x,1e-10))
            else:fz=abs
        qx,qy,I = self.df_G[['qx','qy','I']].values.T
        if not len(caxis)==2:caxis = [fz(I).min(),fz(I).max()]
        dsp.stddisp(scat=[qx,qy,ms,fz(I)],labs=['$q_x$','$q_y$'],
            title='$z=%d\AA$' %self.thick,caxis=caxis,
            pOpt=pOpt,**kwargs)

    def set_I(self,thick):
        self.thick=thick
        b = self.load_b()
        self.df_G.I=0
        for b0 in b:
            b0.set_thickness(thick)
            self.df_G.loc[b0.df_G.index,'I'] += b0.df_G.I.values
        self.df_G.loc[str((0,0)),'I'] = 0

t = np.linspace(0,np.pi,91)
u = np.array([np.cos(t),np.sin(t)]).T
pattern = np.array([[0.2,0.1,1],[0.1,0.3,2]])
pp = {'a':5,'b':5,'alpha':90,'pattern':pattern,'interp':False,'fract':True}
bargs = {'crys':pp,'keV':200,'thick':500,'Nmax':20,'Smax':0.025,'solve':1}

for pp_type in cg.pp_types[2:]:
    pp['pp_type']=pp_type
    bargs['crys']=pp
    b2d = bloch2D_Cont(u,tag=pp_type,path='dat',run=True,bargs=bargs)

    thicks = np.arange(5,101,5)
    # thicks = np.hstack([np.arange(5,26,5),np.arange(30,101,10), np.arange(120,201,20),np.arange(250,501,50)])
    for thick in thicks:#[:2]:
        b2d.show_I(thick=thick,fz='log',xylims=4,
            name='figures/cont2D_%s_z%d.png' %(pp_type,thick),opt='cs')


# fig,ax = dsp.stddisp(opt='')
# for b0 in b: b0.show_ewald(ax=ax,legOpt=0,opt='')
# fig.show()



import importlib as imp
from utils import*
import multislice.multislice as mupy        ;imp.reload(mupy)
import multislice.postprocess as pp         ;imp.reload(pp)
# import matplotlib.animation as animation
plt.close('all')

name ='../dat/Carbone_SC/'
fig_path = '../docs_fig/'

def run(Nz, *kwargs):
    return mupy.Multislice(name,tail='test',data='Carbone_SC001A.dat',
        mulslice=1,keV=200,TDS=False,
        NxNy=1024,slice_thick=1.0,repeat=[8,8,Nz],
        Nhk=10,hk_sym=1,**kwargs)
multi = run(Nz=100, opt='dsrp',fopt='f',ppopt='wB')
# multi.print_log()
# multi.beam_vs_thickness(cm='seismic')
# beams = multi.get_beams()
# multi.gif_beams()


def run_tilts(tilts,Nz,**kwargs):
    mupy.sweep_var(name,'tilt',np.deg2rad(tilts)*1000,do_prev=0,
        mulslice=1,keV=200,TDS=False,
        NxNy=1024,slice_thick=1.0,repeat=[8,8,Nz],
        Nhk=10,hk_sym=1,**kwargs)


def integrate(df_path,tilts,Nz,**kwargs):
    df = pd.read_pickle(df_path)
    iBs,iZs = np.arange(5),np.arange(100,Nz,100)
    nbs,nzs = iBs.size,iZs.size
    I,Iint = np.zeros((nts,nbs,nzs)), np.zeros((nbs,Nz))
    for it,f in enumerate(df.index) :
        multi = pp.load_multi_obj(name+f)
        hks,t,Ib = multi.get_beams()[[0,1,-1]]
        I[it,:,:] = Ib[iBs,:][:,iZs]
        Iint[iBs,:] += Ib[iBs,:]

    hks  =[hk[1:-1].split(',') for hk in hks]
    hk = ['(%d,%d)' %((int(hk[0])/8,int(hk[1])/8)) for hk in hks]
    cs = dsp.getCs('jet',nbs)
    for iz,iZ in enumerate(iZs):
        tle = 'z=$%.0f \AA$' %t[iZ]
        plts = [[tilts[:,0],I[:,iB,iz],[cs[iB],'-o'],'%s' %hk[iB]] for iB in range(nbs) ]
        dsp.stddisp(plts,labs=[r'$\theta(^{\circ})$','$I_b$'],title=tle,lw=2,
            opt='ps', name=fig_path+'SC_tilt.svg')

    plts = [[t,Iint[iB,:],[cs[iB],'-'],'%s' %hk[iB]] for iB in range(nbs) ]
    dsp.stddisp(plts,labs=[r'$z(\AA)$','$I_b$'],lw=2,**kwargs)

# nts=20
# Nz=1000
# tilts = np.vstack([np.linspace(-0.1,0.1,nts),0*np.ones((nts))]).T
# multi = run_tilts(tilts,Nz, opt='dsr',fopt='f',ppopt='',ssh='tarik-CCP4home',v=1)
# df = pp.update_df_info(name+'df.pkl',files=['beams'])
# integrate(df_path=name+'df.pkl',tilts=tilts,Nz=Nz,opt='p',name=fig_path+'SC_Iz_int.svg')
# multi = pp.load_multi_obj(name+'Carbone_SC_tilt10_mulslice.pkl')
# multi.beam_vs_thickness(cm='jet',iBs=range(5),opt='p',name=fig_path+'SC_Iz.svg')


#### load and run an individual simu
# multi = pp.load_multi_obj(name+'Carbone_SC_tilt016_mulslice.pkl')
# multi.run(ssh_alias='tarik-CCP4home')
# multi.ssh_get(ssh_alias='tarik-CCP4home','beams')
# beams = multi.get_beams()
# multi.beam_vs_thickness()



from utils import*
from EDutils import viewers as vw;imp.reload(vw)
from utils import pytest_util    ;imp.reload(pytest_util)

@pytest_util.add_link(__file__)
def test_show_exp():
    v= vw.Base_Viewer('../blochwave/out/test_continuous/tiff',i=10,pargs={'opt':''})
    return v.fig,v.ax
# test_show_exp()



from utils import*
from blochwave import bloch2D as bl          ;imp.reload(bl)
from multislice import mupy_utils as mut     ;imp.reload(mut)
import multislice.multi_2D as ms             ;imp.reload(ms)
plt.close('all')
opts = 'BM' #(Bloch) (Multislice)

# K = 5.024937810560445
# keV = cst.lam2keV(1/K)
keV=200

fig,axB=dsp.create_fig()
cmap  ='jet'
thick = 100
eps   = 0.1

#### Bloch
if 'B' in opts:
    file = 'dat/p1.py'
    # p1 = mut.import_wallpp(file)
    # p1.plot_unit_cells(opts='uAa',nh=3,nk=3)
    b0  = bl.Bloch2D(file,keV=keV,u=[0,1],thick=thick,eps=eps,
        Nmax=5,Smax=0.1,solve=1)
    # b0.show_beams(opts='B',fz=np.abs)
    # b0.show_ewald()
    # b0.show_beams(opts='SVB')
    b0.show_beams_vs_thickness(thicks=(0,thick,1000),strong=['I'],m={'I':1e4},
        linespec='--',marker='',cm=cmap,ax=axB,opt='')
    # b0.G('Sw')
    # idx = b0.get_Istrong(Icols=['I'],m={'I':1e5},out=1)
    # ibs = b0.get_refl(idx)

#multislice
if 'M' in opts:
    ax,bz = b0.crys.params[:2]
    dz=bz/1
    Nx,Nz=5,int(thick/dz)
    x,z,f = b0.crys.get_potential_grid_p1(ndeg=2**6)

    tilt = 0#np.linspace(0,0.08,40)[0]
    ms0 = ms.Multi2D([x,z,f],ax,bz,keV=keV,Nx=Nx,nz=Nz,
            dz=dz,eps=eps,sg=-1,
            iZs=1,iZv=np.inf,tilt=tilt)
    # ms0.V_show()
    # ms0.Qz_show([-1],opts='nO',xylims=['x',-6,6],lw=2)
    ms0.Bz_show(np.arange(-1,2)*Nx,lw=2,cmap=cmap,
        ax=axB,opt='',xylims=[0,thick,0,1])

fig.show()



from utils import*
from crystals import Crystal
from blochwave import bloch as bl;imp.reload(bl)
from scattering import scattering_factors as scatf  ;imp.reload(scatf)
from utils import pytest_util
import pytest,os,copy
plt.close('all')
keV=200

try:
    out,ref,dir = pytest_util.get_path(__file__)
    path=out+'/felix/'
    b = bl.Bloch(cif_file='GaAs',u=[-1,1,0],keV=keV,Nmax=10,Smax=0.2,path=out,solve=0)

    cif='GaAs_felix.cif'
    if not os.path.exists(path+'eigenvals.txt'):
        bf=copy.copy(b)
        bf._solve_Felix(cif,nbeams=200,thicks=(10,20,10))

    A   = np.loadtxt(path+'eigenvals.txt')
    hkl = np.array(A[:,:3],dtype=int)
    hkl_str = [str(tuple(h)) for h in hkl]
    b.solve(hkl=hkl,Smax=0,Nmax=10)#Smax=0.02,Nmax=7)
    # idx = b.get_beam(refl=hkl_str)
except Exception as e:
    print(e)
    pass


@pytest.mark.new
@pytest_util.cmp_ref(__file__)
def test_solve_felix():
    bf=copy.copy(b)
    bf._solve_Felix(cif,nbeams=200,thicks=(10,250,10))
    return bf.gammaj

@pytest.mark.lvl2
def test_coords():
    dfa = pd.read_csv(path+'atoms.txt',sep="  *",names=['Z','x','y','z'],engine='python')
    crys = b.crys
    xyz  = pd.DataFrame()
    xyz['Z'] = np.array([a.atomic_number for a in crys.atoms])
    xyz[['x','y','z']] = [a.coords_cartesian for a in crys.atoms] #,columns=['x','y','z'])

# @pytest.mark.lvl2
# def test_scattering_factors():
#     abcd = np.load('/home/tarik/Documents/git/ccp4/src/electron-diffraction/scattering/data/abcd.npy')
#     aGa,aAs = abcd[[31,33]]
#     a_Ga = np.array([6.255284640e-01,1.100056500e-01,2.053029010e00,2.410957860e00,2.896081200e-01,4.786857360e01,
#       2.079105940e-01,3.278072240e-01,3.450796170e-01,7.431390610e-01,6.556342980e-03,3.094113690e-02])
#     a_As = np.array([7.778752180e-01,1.507331570e-01,5.938481500e-01,1.428822090e02,1.959187510e00,1.747503390e00,
#       1.798802260e-01,3.318008520e-01,8.632672220e-01,5.854902740e00,9.590534270e-03,2.337775690e-02])
#     print(aGa-a_Ga, aAs-a_As)

@pytest.mark.lvl1
def test_structure_factor():
    df  = pd.read_csv(path+'StructureFactors.txt',sep="  *",names=['h','k','l','qx','qy','qz','Fr','Fi'],engine='python')
    df.index=[str(tuple(hkl)) for hkl in df[['h','k','l']].values]
    df=df.loc[hkl_str] #

    df['Rg'] = np.linalg.norm(df[['qx','qy','qz']].values,axis=1)
    df['q']  = df.Rg/(2*np.pi)
    df['qb'] = b.df_G.q.loc[hkl_str]
    # Za = np.array(b.pattern[:,-1],dtype=int)
    # fj = scatf.get_fe(Za,df.q)

    df['Ug'] = b.df_G.loc[df.index,'Vg']
    # df['Ug'] = np.sum(fj,axis=1)
    df['Ufelix'] = df.Fr+1J*df.Fi
    df['Uf'] = abs(df.Ufelix)
    df['Udiff'] = abs(df.Ug-df.Ufelix)#/df.Uf
    # print(b.df_G.loc[df.index,['qx','qy','qz']])
    # print(df.iloc[:20][['Rg','Ufelix','Ug']])
    print(df.iloc[:20][['Ufelix','Ug','Udiff']])
    idx,cols = (df.Udiff>0.01) &(df.Uf>1e-8),['Ufelix','Ug','Udiff','Uf']
    print(df.loc[idx,cols])
    dmax=df.Udiff.max()
    print(dmax)
    assert dmax<1e-4

def get_H():
    H = np.loadtxt(path+'matrix.txt')/100
    H = H[:,3::2]+1J*H[:,4::2]
    return H

@pytest.mark.lvl1
def test_excitation():
    H=get_H()
    dfM = pd.DataFrame(hkl_str,columns=['hkl'],index=hkl_str)
    dfM['Sw_felix'] = np.diag(H).real
    dfM['Sw_bloch'] = b.df_G.loc[hkl_str]['Sw']*(2*np.pi)
    print('mean excitation diff:',abs(dfM.Sw_felix-dfM.Sw_bloch).mean())
    dmax = abs(dfM.Sw_felix-dfM.Sw_bloch).max()
    print(dfM);print(dmax)
    assert dmax<1e-3


@pytest.mark.lvl1
def test_matrix():
    H=get_H()
    Hdiff = abs(b.H-H)
    print('mean matrix error :' ,Hdiff.mean())
    print('max matrix error :' ,Hdiff.max())
    assert Hdiff.max()<1e-3

def get_eigen():
    g = np.loadtxt(path+'eigenvals.txt')
    C = np.loadtxt(path+'eigenvec.txt')
    g = g[:,3::2]+1J*g[:,4::2];g=g[:,0]
    C = C[:,3::2]+1J*C[:,4::2]
    return g,C


@pytest.mark.lvl1
def test_eigen():
        #felix
        g,C = get_eigen()
        idx_g  = np.argsort(g)
        g = g[idx_g]
        C = C[:,idx_g]
        #bloch
        idx_b  = np.argsort(b.gammaj)
        gammaj = b.gammaj[idx_b]
        Cj     = b.CjG[:,idx_b]

        # print('Cj.gamma.Cjdag',np.linalg.norm(Cj.dot(np.diag(gammaj)).dot(np.linalg.inv(Cj))-b.H))
        # print('C.g.Cdag',np.linalg.norm(C.dot(np.diag(g)).dot(np.linalg.inv(C))-H))
        dg = (abs(g-gammaj)/abs(g))
        dfg = pd.DataFrame(np.array([g,gammaj,dg]).T,columns=['felix','bloch','diff'])
        print('mean eigen val error percentage :',dg.mean())
        print('max eigen val error percentage :' ,dg.max())
        print(dfg)
        assert dg.mean()<5e-2

@pytest.mark.lvl1
@pytest_util.add_link(__file__)
def test_intensities():
        with open(path+'felix.inp','r') as f:l=[l.strip() for l in f.readlines()]
        z0,z1,dz=[float(s.split("=")[1]) for s in l if 'Thickness' in s]
        z = np.arange(z0,z1+1,dz)
        I = np.loadtxt(path+'intensities.txt')[:,3:]
        idx = np.arange(1,10)
        I=I[idx]            #;print(I.shape)
        #### force calculations with felix eigen solutions
        # g,C=get_eigen()
        # b.gammaj=g
        # b.CjG=C
        # print(g.shape,C.shape)
        # b.invCjG=np.linalg.inv(C)

        b._set_beams_vs_thickness(thicks=z)
        Ib = b.get_beams_vs_thickness(idx=idx)
        # Rfz = np.abs(np.abs(I).sum(axis=0)-np.abs(Ib).sum(axis=0))/np.abs(Ib).sum(axis=0)
        # dsp.stddisp([z,Rfz,'b-'],labs=['$z$','Rf'])
        # print('max Rfactor error :',np.max(Rfz))
        # idI = I>1e-4
        print(Ib.shape)
        dI = np.abs(Ib-I)
        print('error in I : max=%.2f, mean=%.2E' %(dI.max(),dI.mean()))
        # print('diff with ref:',abs(Rfz-np.load(path+'Rfz.npy')).sum())

        zA = 250

        idz=abs(z-zA).argmin()#;print(idz)
        dI = np.array([(np.abs(I1-I0)[I0>1e-3]/I0[I0>1e-3]).max() for I0,I1 in zip(I[:,:idz],Ib[:,:idz])])
        print(dI)
        print('max err I over %dA : %.2E' %(zA,dI.max()))
        # assert dI.max()<0.05

        cs = dsp.getCs('jet',I.shape[0])
        plts = [[z,If,[c,'--o'],''] for If,c in zip(I,cs)]
        plts += [[z,Ib,[c,'-x'],''] for Ib,c in zip(Ib,cs)]
        legElt = {'bloch':'k-x','felix':'k--o'}
        return dsp.stddisp(plts,lw=2,legElt=legElt,
            xylims=['x',0,zA],opt='p')

# test_excitation()
# test_matrix()
# test_structure_factor()
# test_eigen()
# test_solve_felix()
# fig,ax=test_intensities();fig.show()



from utils import*
import importlib as imp
import wallpp.plane_group as pg   ;imp.reload(pg)
import nearBragg.nearBragg as nb ;imp.reload(nb)
plt.close('all')
path='../../multislice/docs_fig/multi2D/'
opts='I'

# Problem
keV   = 200        # wavelength 200keV
Nx    = 10          # Transverse unit cells
Nz    = 20         #
# ndeg  = 2**8       # number of pixels
pptype,ax,bz,angle,Za = 'p1',10,5,90,  2
pattern = np.array([[5,2.5,Za]])
eps = 1
opt='p'
method = 'D'


q0s = np.linspace(0,1,401)
nbG = nb.NearBragg(pattern.T,ax,bz,keV=keV,Nx=Nx,Nz=Nz,eps=eps,
    method=method,q0s=q0s,iZv=20)
nb0 = nb.NearBragg(pattern.T,ax,bz,keV=keV,Nx=Nx,Nz=Nz,eps=eps,
    method='G',q0s=q0s,iZv=100)
nbG.F_show(qopt=0,lw=2)
fig,ax = dsp.stddisp([nbG.q0s,nbG.I,'bo-'],opt='')
dsp.stddisp([nb0.q0s,nb0.I,'ro--'],ax=ax)

if method=='P':
    z = bz*np.arange(Nz+1)/10
    sig_e = nbG.sig_e
    le = ax*bz/sig_e/10
    Pcoh = np.exp(-z/le)
    Pkin = (z/le)*np.exp(-z/le)
    Pdyn = 1-Pcoh-Pkin


    cs = dsp.getCs('Spectral',3)
    plts  = [[z,I,cs[i],'%d' %i] for i,I in enumerate(nbG.I)]
    plts += [[z,Pcoh,[cs[0],'--'],''],[z,Pkin,[cs[1],'--'],''],[z,Pdyn,[cs[2],'--'],'']]
    plts += [[z,nbG.I.sum(axis=0),'k:']]
    dsp.stddisp(plts,labs=['$z(nm)$','$Proba$'],lw=2,
        opt=opt,name=path+'NBproba.svg')


# iZ = slice(1,-1,100)
# nz = z[iZ].shape[0]
# cs = dsp.getCs('jet',nz)
# plts  = [[q0s,S,cs[i],'z=%dnm' %z[iZ][i]] for i,S in enumerate(nbG.S[iZ,:,2])]
# dsp.stddisp(plts,labs=['$q(\AA^{-1})$','Double scattering area$(\AA)$'],lw=2,
#     opt=opt,name=path+'NBdouble.svg')
#
# cs = dsp.getCs('jet',nz)
# plts  = [[q0s,S,cs[i],'z=%dnm' %z[iZ][i]] for i,S in enumerate(nbG.S[iZ,:,1])]
# dsp.stddisp(plts,labs=['$q(\AA^{-1})$','Single scattering area$(\AA)$'],lw=2,
#     opt=opt,name=path+'NBsingle.svg')



from utils import*                  ;imp.reload(dsp)
from EDutils import viewers as vw   ;imp.reload(vw)

# plt.close('all')

# v = vw.Viewer(config='dat/config.pkl')
v = vw.Viewer(config='dat/4_2beam_config.pkl')

# v = vw.Viewer(cif_file='diamond',orient=[0]*2+[0.1]*2,
#     Smax=0.025,Nmax=8,thick=250,path='dat',
#     init_opts='R',pets_opts='S',xylims=3)



import sys,numpy,os
from utils import*
from crystals import Crystal
import multislice.multislice as mupy
import multislice.postprocess as pp
import multislice.rotating_crystal as rcc
import importlib as imp
imp.reload(mupy)
imp.reload(pp)
imp.reload(rcc)

plt.close('all')
path = '../dat/biotin/'
file = path+'biotin.cif'

# rcc.show_cell(file,x0=-1)

def create_xyz(opts='gps'):
    nz = np.array([1,2,1,1,1,1,1,1 ,1,1 ])#,1)
    nx = np.array([0,1,1,2,3,4,6,8 ,12,24])#,1,7,12,24])
    angles = np.arctan(nx/(4*nz))*180/np.pi
    data = ['biotin%d0%d.xyz' %(n,m) for n,m in zip(nx,nz)]
    if opts:
        # rcc.import_cif(file,path+'biotin001.xyz')
        Nx,Nz=[(1,1),(30,5)]['g' in opts]
        rcc.rotate_xyz(file,nx,nz,Nx=Nx,Nz=Nz,opt=opts)
    for dat,a in zip(data,angles) : rcc.show_grid(path+dat,title=dat+' %.1f' %a)
    print('angles:',angles)
    return data,angles

def run_main(dfname='df0.pkl',thick=10,ssh='',hostpath=''):
    cols  = ['host','state','dat','angle','tilt']
    df    = pd.DataFrame(columns=cols+pp.info_cols)
    Nx = np.array(np.round(lats[:,0].max()/lats[:,0]),dtype=int)
    Ny = np.array(np.round(Nx*lats[:,0]/lats[:,1]),dtype=int)
    Nz = np.array(np.round(thick/lats[:,2]),dtype=int)
    for iD,dat in enumerate(data):
        istr  = ('m%d' %iD).zfill(2)
        multi = mupy.Multislice(path,data=data[iD],tail=istr,
            mulslice=False,keV=200,#tilt=[tx*np.pi/180/1000,0],
            NxNy=2048,slice_thick=1.0,Nhk=5,repeat=[Nx[iD],Ny[iD],Nz[iD]],
            #TDS=True,T=300,n_TDS=15,
            opt='sr',fopt='f',v='nctr',#nctrdDR',
            ssh=ssh,hostpath=hostpath
            )
        df.loc[multi.outf['obj']] = [np.nan]*len(df.columns)
        df.loc[multi.outf['obj']][cols] = [ssh,'start',data[iD],angles[iD],0]
    df.to_pickle(path+dfname)
    print(green+'Dataframe saved : '+yellow+path+dfname+black)

def run_simus(thick=5000,nts=91,ssh=''):
    cols  = ['host','state','dat','angle','tilt']
    df    = pd.DataFrame(columns=cols+pp.info_cols)
    tilts = np.linspace(0,90,nts)
    Nx = np.array(np.round(lats[:,0].max()/lats[:,0]),dtype=int)
    Ny = np.array(np.round(Nx*lats[:,0]/lats[:,1]),dtype=int)
    Nz = np.array(np.round(thick/lats[:,2]),dtype=int)
    for i in range(nts):
        pad   = int(np.log10(nts))+1
        istr  = ('%d' %i).zfill(pad)
        iD    = np.abs(tilts[i]-angles).argmin()
        tx    = tilts[i]-angles[iD]
        multi = mupy.Multislice(path,data=data[iD],tail=istr,
            mulslice=False,keV=200,tilt=[tx*np.pi/180/1000,0],
            NxNy=512,slice_thick=1.0,Nhk=5,repeat=[Nx[iD],Ny[iD],Nz[iD]],
            #TDS=True,T=300,n_TDS=15,
            opt='sr',fopt='f',v='nctr',#nctrdDR',
            ssh=ssh,
            )
        df.loc[multi.outf['obj']] = [np.nan]*len(df.columns)
        df.loc[multi.outf['obj']][cols] = [ssh,'start',data[iD],tilts[i],tx]
    df.to_pickle(path+'df.pkl')
    print(green+'Dataframe saved : '+yellow+path+'df.pkl'+black)

        # multi.wait_simu(ssh_alias='tarik-CCP4home')
        # multi.print_log()
        # multi.postprocess(ppopt='uwP',ssh_alias='tarik-CCP4home')

def run_tilts(**kwargs):
    tx = np.linspace(0,0.1,10)#deg
    tilts = [[0,t*np.pi/180*1000] for t in tx]
    mupy.sweep_var(path+'tilts/','tilt',tilts,tail='m',df='dfm.pkl',do_prev=0,**kwargs)


def update_patterns(dfname='df.pkl',ssh='',hostpath=''):
    df = pp.update_df_info(path+dfname,hostpath=hostpath)
    for dat in df.index:
        multi=pp.load_multi_obj(path+dsp.dirname(dfname)+dat)
        multi.ssh_get(ssh,'pattern',hostpath=hostpath)
        multi.save_pattern()
    return df

def get_figs(dfname='df.pkl'):
    df = pd.read_pickle(path+dfname)
    cs = dsp.getCs('Blues',3)#,dsp.getCs('Reds'),dsp.getCs('Greens')
    plts = [[],[],[]]
    #pad = int(np.log10(df.index.size))+1
    for i,dat in enumerate(df.index):
        multi=pp.load_multi_obj(path+dsp.dirname(dfname)+dat)
        multi.pattern(tol=1e-4,Iopt='Insgl',gs=0.25,caxis=[-6.3,0],Nmax=500,
            cmap='binary',imOpt='hc',#rings=[0.1,0.2],#,1],,
            pOpt='Xt',xyTicks=1,xylims=[-3,3.01,-3,3.01],#xylims=[-2,2.01,-2,2.01],
            opt='s',name=path+'figures/%s_pattern.png' %dat.replace('.pkl',''))
        # multi.beam_vs_thickness()
    #     for j in range(len(hk)):plts[jq]+=[[t,I[j,:],cs[i],'']]
    # for j in range(3):
    #     dsp.stddisp(plts[j],title='hk=%s' %hk[j],
    #         imOpt='ch',cmap='Blues',caxis=[0,90],pOpt='tG')


# data,angles = create_xyz(opts='ps')
# lats  = np.array([rcc.show_grid(path+dat,opt='')[0] for dat in data])
# multi = run_main(thick=2000,ssh='badb',hostpath='/data3/lii26466/multislice/biotin/')

# multi = run_simus(thick=100,nts=3,ssh='')#'tarik-CCP4home')
multi = run_tilts(keV=200,mulslice=False,
    NxNy=2048,slice_thick=1.0,Nhk=5,repeat=[2,1,100],
    opt='dsr',fopt='f',v=1,
    ssh='badb',cluster=1,hostpath='/data3/lii26466/multislice/biotin/tilts/')

# df = update_patterns('tilts/df.pkl',ssh='badb',hostpath='/data3/lii26466/multislice/biotin/tilts/')
# get_figs(dfname='tilts/df.pkl')

# df = pp.update_df_info(path+'df0.pkl',hostpath='/data3/lii26466/multislice/biotin/')
# df = update_patterns('df0.pkl',ssh='badb',hostpath='/data3/lii26466/multislice/biotin/')
# get_figs(dfname='df0.pkl')



import blochwave,time,sys,pytest,os,copy
from pytest_html import extras
from utils import*                  ;imp.reload(dsp)
from utils import pytest_util       ;imp.reload(pytest_util)
from blochwave import bloch         ;imp.reload(bloch)
from blochwave import util as bloch_util;imp.reload(bloch_util)
plt.close('all')

out,ref,dir = pytest_util.get_path(__file__)
b0 = bloch.Bloch('diamond',path=out,keV=200,u=[0,0,1],Nmax=8,Smax=0.05,
    opts='svt',thick=100)

@pytest_util.cmp_ref(__file__)
def test_solve_bloch():
    g=b0.gammaj
    return g

@pytest_util.cmp_ref(__file__)
def test_beam_thickness():
    idx   = b0.get_beam()
    return b0.get_beams_vs_thickness(dict_opt=False,idx=idx)

@pytest_util.add_link(__file__)
def test_show_beam_thickness():
    return b0.show_beams_vs_thickness(thicks=(0,200,1000),
        beam_args={'cond':'(Vga>1e-4) & (Sw<1e-2)'},cm='jet',
        opt='')

# @pytest.mark.new
@pytest_util.add_link(__file__)
def test_convert2tiff():
    import tifffile
    tiff_file=out+"/I.tiff"
    v=b0.convert2tiff(tiff_file=tiff_file,
        show=False,cutoff=10,thick=200,Imax=1e7)
    im=tifffile.imread(tiff_file)
    return dsp.stddisp(im=[im],
        cmap='gray',caxis=[0,20],pOpt='t',opt='')

@pytest_util.add_link(__file__)
def test_show_beams():
    return b0.show_beams(cmap='viridis',opt='')#mag=1000,)

@pytest_util.add_link(__file__)
def test_show_Idyn_vs_Ikin():
    b0 = bloch.Bloch('diamond',path=out,keV=200,u=[2,4,1],Nmax=8,Smax=0.05,
        opts='svt',thick=100,thicks=(0,1000,1000))
    return b0.show_Idyn_vs_Ikin(iZs=[10,100,500,999],opt='')#slice(249,None,250))



@pytest.mark.lvl2
def test_set_beam():
    b0.set_beam(keV=200)
    b0.set_beam(u=[1,0,1])
    b0.set_beam(K=1/cst.keV2lam(100)*np.array([1,0,0]))

@pytest.mark.lvl2
def test_get_beam():
    b0.df_G.index = np.arange(b0.df_G.index.size)
    idx = b0.get_beam(cond={'tol':1e-2})
    idx = b0.get_beam(refl=[str((0,0,0))])
    hkl = b0.get_beam(refl=[str((0,0,0))],index=False)
    idx = b0.get_beam()

def test_beam_thickness():
    # b0.__dict__.pop('Iz')
    b0.get_beams_vs_thickness(dict_opt=True)

def test_gets():
    b0.get_Xig()
    b0.get_Sw()
    print('  0,0,0 in bloch : ' ,b0.is_hkl(0,0,0))
    print('100,0,0 in bloch : ' ,b0.is_hkl(100,0,0))

@pytest.mark.lvl2
def test_show_Fhkl():
    b0.show_Fhkl(xylims=6,h3D=1,ms=30,cmap='Greens',opt='')
    b0.show_Fhkl(s=np.s_[:,:,3],xylims=6,ms=30,cmap='Greens',opt='')
    b0.show_Fhkl(s='h=0',opts='l',xylims=7,ms=50,cmap='Greens',pOpt='im',caxis=[0,1.5],opt='')

# @pytest.mark.slow
@pytest_util.add_link(__file__)
def test_bloch_convergence():
    b=copy.copy(b0)
    refl=b.get_beam(cond={'tol':1e-10,'opt':''},index=False)#;print(idx)
    # Smax,Nmax=(0.05,0.1,0.2,0.5),(6)
    Smax,Nmax=(0.1),(6,8,10,12)
    b.convergence_test(Smax=Smax,Nmax=Nmax)#,hkl=refl)
    return b.show_convergence(hkl=refl,xlab='Nmax',opt='')

def test_load_bloch():
    b = bloch_util.load_bloch(file=out+'/diamond001_200keV_bloch.pkl')
    bloch_util.load_bloch(path='.')
    bloch_util.load_bloch(path=out)
    bloch_util.load_bloch(path=out,tag='001')

# test_load_bloch()
# print(b0.get_beam())
# test_get_beam()
# test_gets()
# test_beam_thickness()
# test_show_beams()
# test_show_Idyn_vs_Ikin()
# test_bloch_convergence()

#test_beam_thickness()
# b0 = test_bloch_solve()
# test_set_beam(b0)
# test_show_Fhkl(b0)
# test_convert2tiff()
# b = bloch_convergence(n=6)



from utils import*
from wallpp import config as cg      ;imp.reload(cg)
from wallpp import wallpaper as wp   ;imp.reload(wp)
plt.close('all')
fig_path = 'figures/'


pattern = [[0.2,0.1,1],[0.1,0.3,2]]
pp_args = {'a':2,'b':3,'alpha':90,'pattern':pattern,
    'fract':True,'interp':False,'nh':3,'nk':3}

wallpps={}
fz = lambda x :np.log10(np.maximum(abs(x),1e-5))
for i,pp_type in enumerate(cg.pp_types):
    pp = wp.Wallpaper(pp_type,**pp_args)
    fig,(ax1,ax2) = dsp.create_fig('f',rc='12',pad=5)
    pp.plot('uag',ax=ax1,pOpt='tX',labs=['$x$','$y$'],xylims=[0,5,0,5],
        ms=50,opt='')
    pp.show_structure_factor(title=pp_type,fz=fz,
        caxis=[-5,0],ax=ax2,pOpt='tX',xylims=5,
        name=fig_path+'sf_%s_%s.png' %(str(i).zfill(2),pp_type),opt='sc')
    wallpps[pp_type] = pp



# -*- coding: utf-8 -*-

'''
Math extension for Python-Markdown
==================================

Adds support for displaying math formulas using [MathJax](http://www.mathjax.org/).

Author: 2015, Dmitry Shachnev <mitya57@gmail.com>.
'''

import markdown

class MathExtension(markdown.extensions.Extension):
    def __init__(self, *args, **kwargs):
        self.config = {
            'enable_dollar_delimiter': [False, 'Enable single-dollar delimiter'],
        }
        super(MathExtension, self).__init__(*args, **kwargs)

    def extendMarkdown(self, md, md_globals):
        def handle_match_inline(m):
            node = markdown.util.etree.Element('script')
            node.set('type', 'math/tex')
            node.text = markdown.util.AtomicString(m.group(3))
            return node

        def handle_match(m):
            node = markdown.util.etree.Element('script')
            node.set('type', 'math/tex; mode=display')
            if '\\begin' in m.group(2):
                node.text = markdown.util.AtomicString(m.group(2) + m.group(4) + m.group(5))
            else:
                node.text = markdown.util.AtomicString(m.group(3))
            return node

        configs = self.getConfigs()
        inlinemathpatterns = (
            markdown.inlinepatterns.Pattern(r'(?<!\\|\$)(\$)([^\$]+)(\$)'),  #  $...$
            markdown.inlinepatterns.Pattern(r'(?<!\\)(\\\()(.+?)(\\\))')     # \(...\)
        )
        mathpatterns = (
            markdown.inlinepatterns.Pattern(r'(?<!\\)(\$\$)([^\$]+)(\$\$)'), # $$...$$
            markdown.inlinepatterns.Pattern(r'(?<!\\)(\\\[)(.+?)(\\\])'),    # \[...\]
            markdown.inlinepatterns.Pattern(r'(?<!\\)(\\begin{([a-z]+?\*?)})(.+?)(\\end{\3})')
        )
        if not configs['enable_dollar_delimiter']:
            inlinemathpatterns = inlinemathpatterns[1:]
        for i, pattern in enumerate(inlinemathpatterns):
            pattern.handleMatch = handle_match_inline
            md.inlinePatterns.add('math-inline-%d' % i, pattern, '<escape')
        for i, pattern in enumerate(mathpatterns):
            pattern.handleMatch = handle_match
            md.inlinePatterns.add('math-%d' % i, pattern, '<escape')

def makeExtension(*args, **kwargs):
    return MathExtension(*args, **kwargs)



from utils import*                  ;imp.reload(dsp)
from EDutils import utilities as ut ;imp.reload(ut)
from blochwave import bloch_pp as bl;imp.reload(bl)
from utils import pytest_util
import pytest,os
plt.close('all')

tag='big' #do not change as it messes up with the cmp_ref
npts={'big':50,'small':5}[tag]

@pytest.mark.slow
def test_rock():
    uvw = ut.get_uvw_rock(e0=[0,0,1],e1=[2,1],deg=3,npts=npts,show=0)
    rock = bl.Bloch_cont(path=out,tag=tag,uvw=uvw,Sargs=bloch_args)

try:
    out,ref,dir = pytest_util.get_path(__file__)
    bloch_args = {'cif_file':'diamond','keV':200,'thick':200,'thicks':(0,200,500),
        'Nmax':9,'Smax':0.025,'solve':True,'opts':'stz'}
    rock_file=file=out+'/rock_%s.pkl' %tag
    if not os.path.exists(rock_file):test_rock()
    rock = ut.load_pkl(file=rock_file)
except Exception as e:
    print(e)
    pass

def test_load():
    rock.load(0)

@pytest.mark.lvl1
@pytest_util.add_link(__file__)
def test_show_tiff():
    rock.convert2tiff(thick=200,Imax=1e6,aperpixel=0.01)
    vw=rock.show_tiff(cutoff=40,frame=2,pargs={'opt':''},
        h=False)
    return vw.fig,vw.ax

@pytest.mark.lvl1
@pytest_util.add_link(__file__)
def test_plot_rocking_cond():
    rock.set_beams_vs_thickness(thicks=(0,100,100))
    cond=lambda dfG:bl.strong_beams(dfG,tol=1e-2,n=5)
    return rock.plot_rocking(cond=cond,opts='',opt='')

@pytest.mark.lvl1
@pytest_util.add_link(__file__)
def test_plot_rocking_quick():
    return rock.plot_rocking(refl=[str((2,2,0))],zs=[50,100],opts='',opt='')

# @pytest.mark.lvl1
@pytest_util.cmp_ref(__file__)
def test_integrate_rocking():
    refl,h=rock.get_beams(cond=lambda dfG:bl.strong_beams(dfG,tol=1e-2,n=5))
    rock._integrate_rocking(refl=refl,new=1)
    return rock.Iz_dyn[refl[0]]

@pytest.mark.lvl1
@pytest_util.add_link(__file__)
def test_plot_integrate():
    refl,h=rock.get_beams(cond=lambda dfG:bl.strong_beams(dfG,tol=1e-2,n=5))
    # refl=refl[2:3]
    return rock.plot_integrated(refl=refl,opt='')

# rock._build_index()
# test_rock()
# fig,ax=test_plot_rocking_quick()
# test_integrate_rocking()
# test_plot_integrate()

# rock.plot_rocking(refl=['(-2, -2, 0)','(-2, 2, 0)','(2, 2, 0)','(2, -2, 0)'])
# hkls=bl.strong_beams(rock.load(20).df_G,tol=1e-3,n=5,opt='F')
# print(hkls)
# hkls = ['(-2, -2, 0)','(-2, 2, 0)','(2, 2, 0)','(2, -2, 0)']
# i=0
# df = rock.get_frames(hkls[i])
# z,I = rock.get_rocking(refl=hkls,zs=np.arange(25,100,25))#iZs=slice(1,None,50))#zs=np.arange(25,200,50))
# rock.plot_rocking(refl=hkls[i:i+1],zs=np.arange(25,100,25))
# rock.plot_rocking(refl=[hkls[i]],iZs=[-1])#zs=np.arange(25,100,25))
# rock.plot_rocking(refl=hkls,iZs=[-1],x='Sw')


# cond = '(Sw<2e-2) & (Vga>1e-6) & (I>1e-3)'
# cond = '(Sw<1e-4) & (I>1e-4) '
# fz = lambda x : -np.log10(np.maximum(x,1e-10))
# cond = ''#'(Sw<1e-2) & (Vga>1e-6)'
# hkls = [[-8,-2,2],[-5,1,1],[-1,7,-1],[3,7,-1],[-2,6,0]]
# refl = [[-8,-2,2],[-5,1,1],[-1,-7,1],[3,7,-1],[-2,6,0],[7,-3,-1],[5,1,-1]]
# hkls = [[refl[1]]]
# rock = bl.bloch_rock(tag='diamond_rand',uvw=uvw,
#     omega=omega,bloch_args=bloch_args,
#     thicks=(0,800,400),
#     ts0=1.0,refl=refl,cond=cond,
#     zs=np.arange(1,6)*50,hkls=hkls,
#     Z_args={},#{'new':1},
#     path=path,opts=opts,opt=opt)


# rock = ut.load_pkl('dat/bloch/rock_diamond_rand.pkl')
# b = rock.load(0)
# b.get_beam(cond=cond)



from utils import*                   ;imp.reload(dsp)
from EDutils import utilities as ut  ;imp.reload(ut)
from EDutils import pets as pt       ;imp.reload(pt)
from EDutils import display as EDdisp;imp.reload(EDdisp)
from blochwave import bloch_pp as bl ;imp.reload(bl)

plt.close('all')

opts='z' #Solve(S) r(save rocking curve)
path='dat/bloch_zenodo_fine'
tag=''

frames  = np.arange(10,120)  # frames to simulate
npts    = 10                # number of intermediate simulations
thick   = 330 #A

pets      = pt.Pets('dat/pets/glycine.pts',gen=0)
alpha0,uvw0 = pets.alpha[frames-1],pets.uvw0[frames-1]
# ut.uvw_add_points(uvw0,npts=npts,plot=1)
uvw = ut.uvw_add_points(uvw0,npts=npts,plot=0)
# uvw=uvw0
alpha = np.linspace(alpha0[0],alpha0[-1],uvw.shape[0])
# d_Sw = ut.get_excitation_errors(-pets.K0*uvw0[0],pets.lat_vec1,Nmax=7,Smax=0.02)
# hkl_pets = pets.rpl.loc[pets.rpl.eval('(F==%d)' %(frames[0]) ), ['hkl','F','I']]
# hkl = hkl_pets.hkl.values
# print(d_Sw.loc[hkl])


if 'S' in opts:
    bloch_args = {'cif_file':'dat/pets/alpha_glycine.cif','Smax':0.1,'Nmax':8,
        'solve':1,'thick':thick,'thicks':(0,thick,500),'keV':200}
    rock = bl.Bloch_cont(path=path,tag=tag,uvw=-uvw,Sargs=bloch_args)

if 1:
    rock_file=path+'/rock_%s.pkl' %tag
    rock = ut.load_pkl(file=rock_file)

    i = 11
    pets.show_exp(frames[i],init='rc')
    pets_simu = pt.Pets('dat/pets_simu/glycine.pts',gen=0)
    # pets_simu.show_exp(frames[i])
    vw=rock.show_tiff(sum_opt=True,cutoff=20,i=i+1,pargs={'xylims':[0,516,516,0]})#,pargs={'opt':'p'},h=1)

    # rock.set_beams_vs_thickness(thicks=(0,330,500))
    fbroad=lambda r2:np.exp(-r2**0.8/0.001)
    tiff_args={'fbroad':fbroad,'gs3':0.05,'nX':25,'Imax':5e6,
        'rot':-203.5,'aperpixel':0.00534,'iz':-1,'Nmax':516,
        'tif_writer_args':{'description':"ImageCameraName: timepix"}}
    ##sum images
    if 's' in opts:
        rock.convert2tiff(**tiff_args,nmax=0)
        rock.convert2tiff(n=npts,nmax=0)
        vw=rock.show_tiff(sum_opt=True,cutoff=20,i=i+1,pargs={'xylims':[0,516,516,0]})#,pargs={'opt':'p'},h=1)
    if 'p' in opts:
        pt.make_pets(pts_file='dat/pets_zenodo/glycine.pts',aperpixel=pets.aper,
            alphas=alpha0,ref_cell=[5.08760,11.80920,5.46150,90.00000,111.99200,90.00000],)
    else:
        b = rock.load(i)
        # b.set_beams_vs_thickness(thicks=(0,330,500))
        # b.convert2tiff(tiff_file='test.tiff',show=1,cutoff=50,**tiff_args)

        # vw=rock.show_tiff(cutoff=20,i=i+1,pargs={'xylims':[0,516,516,0]})#,pargs={'opt':'p'},h=1)
        # rock.convert2png(cutoff=20)


    if 'z' in opts:
        refls={
            0:[(4,1,0),(4,1,-1),(5,1,0),(5,1,1),(5,1,2),(-3,-1,2),(-3,-1,1),(3,1,-4),(3,1,-6),(3,1,-7)],
            4:[(-4,-1,0),(0,0,-2),(0,0,-4),(-3,-1,4),(-3,-1,6),(5,1,0)],
            10:[(0,0,-2),(0,0,2),(0,0,4),(0,0,6),(2,0,6),(-5,1,2),(-4,-1,-2)],
            11:[(-5,-1,8)],
        }

        refl = [str(h) for h in refls[i]]
        b = rock.load(i)
        df_pets=pets.rpl.loc[pets.rpl.eval('(F==%d) &(I>2)' %(frames[i]+1) )]
        df_jana=pets_simu.rpl.loc[pets_simu.rpl.eval('(F==%d)' %(frames[i]+1))]

        hkl_pets = df_pets.hkl
        df_pets.index=hkl_pets

        # b.show_beams_vs_thickness(refl=refl,thicks=(0,330,500),cm='jet')
        b._set_I(iZ=-1)

        b.df_G['hkl']=b.df_G.index
        hkl = b.df_G.loc[b.df_G.hkl.isin(hkl_pets),'hkl']
        hkl_jana = df_jana.loc[df_jana.hkl.isin(hkl_pets),'hkl']
        df_jana.index=df_jana.hkl

        b.df_G['Ipets']=np.nan
        b.df_G['Ijana']=np.nan
        b.df_G.loc[hkl,'Ipets']=df_pets.loc[df_pets.hkl.isin(hkl),'I']
        b.df_G.loc[hkl,'Ijana']=df_jana.loc[df_jana.hkl.isin(hkl),'I']

        print(colors.red+'frame %d' %frames[i]+colors.black)
        # print(b.df_G.loc[hkl,['I','Ipets','Ijana']])
        print(np.setdiff1d(hkl_pets,hkl))
        # print(df_pets.loc[np.setdiff1d(hkl_pets,hkl_jana),'I'])

        hkl=refl
        df=b.df_G.copy()
        df=df.drop(str((0,0,0)))
        EDdisp.show_frame(opts='SVkr',mag=500,rot=-205,df_bloch=df,hkl_idx=hkl,
            xylims=np.array([-1,1,1,-1])*2)
        # df=df.loc[hkl]
        # EDdisp.show_frame(opts='Bk',mag=1000,rot=-205,df_bloch=df,hkl_idx=hkl,
        #     xylims=np.array([-1,1,1,-1])*2)



from utils import*                      ;imp.reload(dsp)
import matplotlib.patches as patches,os
from wallpp import wallpaper as wallpp  ;imp.reload(wallpp)
from wallpp import lattice as lat       ;imp.reload(lat)
from wallpp import config as cg         ;imp.reload(cg)
plt.close('all')
figpath = 'figures/'
datpath = 'dat/'

def test_xygrid(new=0,pp_type='p4'):
    if new:
        pattern,lat_vec = np.load(datpath+'cygne_test2_100_%s.npy'%pp_type,allow_pickle=True)
        pattern=np.array(pattern)
        params = lat.params_from_lat_vec(lat_vec)
        pp = wallpp.Wallpaper(pp,path=datpath,pattern=pattern,**params)
        pp.save()

    pp = wallpp.load(file=os.path.join(datpath,pp_type+'_wallpp.pkl'))
    # pp.show_ref(ndeg=1000)
    # x,y = np.linspace(1.25,5,1000),np.linspace(0,3.5,1000)
    x,y = np.linspace(0,1.2,1000),np.linspace(0.5,1.4,1000);
    pp.show_xygrid(x,y,pOpt='Xet')
    # pp.plot('aug',2,3)

def test_wiki_wallpp():
    for i,pp_type in enumerate(cg.pp_types[1:]):
        pattern,lat_vec = np.load(datpath+'%s.npy' %pp_type,allow_pickle=True)
        params = lat.params_from_lat_vec(lat_vec)
        pattern=np.array(pattern).T
        pp = wallpp.Wallpaper(pp_type,pattern=pattern,**params,interp=False)
        pp.plot(opts='aug',nh=2,nk=2,title=pp_type,pOpt='tXe',
            name=figpath+'wiki_%s_%s.png' %(str(i+1).zfill(2),pp_type),opt='sc')

def test_swan_wallpp(params = {'a':1.5,'b':1,'alpha' : 80}):
    # z = np.dot(im[...,:3], [0.2989, 0.5870, 0.1140])
    # from PIL import Image
    # z = Image.open('figures/cygne100.png').convert('LA')

    im = plt.imread(figpath+'cygne_test2_100.png')
    z = np.flipud(im[:,:,2])
    nx,ny = z.shape
    x,y = np.meshgrid(np.arange(nx),np.arange(ny))
    x,y,z = x.flatten()/500-0.12,y.flatten()/500-0.15,z.flatten()
    pattern0 = np.array([x,y,z]).T

    # plt.imshow(im)
    # dsp.stddisp(scat=[x,y,z])
    # plt.show()

    wallpps=dict()
    for i,pp_type in enumerate(['pg']):#cg.pp_types):
        pattern = pattern0.copy()
        pattern[:,:2] += cg.asym_cells[pp_type].mean(axis=0)
        pp = wallpp.Wallpaper(pp_type,pattern=pattern,**params)
        name = figpath+'cygne_%s_%s.png' %(str(i).zfill(2),pp_type)
        pp.plot('uag',nh=2,nk=2,title=pp_type,pOpt='tXe',
            opt='sc',name=name)
        # wallpps[pp_type] = pp


def get_supercell(lat2D,u):
    lat_vec = lat2D.lattice_vectors
    uxy = np.array(u).dot(lat_vec)
    uxy /=np.linalg.norm(uxy)
    sign = lambda x :1 if x>=0 else -1
    nh,nk = -20*sign(uxy[1]),20*sign(uxy[0])
    hk,R = lat2D.get_lattice(nh,nk)
    idx = np.argmin(abs(R.dot(uxy)))

    d = R[idx].dot(uxy)
    m,n = hk[idx]
    R0  = R[idx]
    return R0,d,m,n

def test_mesh():
    pp = wallpp.load(file=datpath+'p3_wallpp.pkl')
    u = [3,1]
    # lat2D = lat.Lattice2D('hex',a=1)
    lat2D = pp.lat
    R,d,m,n = get_supercell(lat2D,u=u)
    # print(d,m,n)

    lat_vec=pp.lat_vec
    v=np.array([m,n]).dot(lat_vec)
    K = np.array(u).dot(lat_vec)
    O1 = -u[0]*lat_vec[0]
    O2 = O1+K
    plts = []
    plts+= [ [ [K[0],0],[K[1],0],'r','K'] ]
    plts+= [ [ [O1[0],O2[0]],[O1[1],O2[1]],'r--',''] ]
    plts+= [ [ [R[0],0],[R[1],0],'b','R'] ]

    # fig,ax = lat2D.plot_lattice(m,n,lw=2,opt='')
    # dsp.stddisp(plts,lw=2,equal=1,ax=ax)



    #mesh
    thick,Tz = 10*pp.a,np.linalg.norm(R)
    x0  = np.linspace(0,thick,1000)
    z0  = np.linspace(0,1.3*Tz,1000)

    #get im for mesh
    x,z  = np.meshgrid(x0,z0)
    xz   = np.array([x.flatten(),z.flatten()]).T
    uv   = np.array([K/np.linalg.norm(K),R/np.linalg.norm(R)])
    xz_  = xz.dot(uv)                               #oriented with respect to u
    xzf_ = xz_.dot(np.linalg.inv(lat_vec))          #fractional coords
    xzf_-= np.floor(xzf_)                           #in first unit cell


    im = np.stack([np.reshape(f(xzf_),x.shape) for f in pp.f],axis=2)
    # x1,z1 = xz_.T
    # x,z = np.reshape(x1.shape),np.reshape(x1.shape)
    # dsp.stddisp(im=[im[:,:,1]],pOpt='tXe')
    hk,Rs = pp.lat.get_lattice(np.arange(-10,11),np.arange(-10,15))
    xR,yR = (Rs.dot(np.linalg.inv(uv))).T
    scat = [xR,yR,50,'b']
    pps = [patches.Rectangle((0,0),600,Tz,alpha=0.5,color='b')]
    dsp.stddisp(scat=scat,patches=pps,im=[im],xylims=[0,x0.max(),0,z0.max()],
        im_args={'origin':'lower','extent':[0,x0.max(),0,z0.max()]})



test_wiki_wallpp()
# test_swan_wallpp()
# test_xygrid(pp_type='p3',new=0)



from utils import *
from wallpp import lattice#;imp.reload(lattice)
# plt.close('all')
lat_types = lattice.lat_types
lats = {lat:lattice.Lattice2D(lat,a=2,b=3,alpha=50) for lat in lat_types }


def plot_lattices() :
    for lat_type,lat in lats.items():
        lat.plot_lattice(nh=6,nk=5, lw=2, equal=1)

def test_lattice_exception():
    try:
        lattice.Lattice2D('rubish')
    except Exception as e:
        print(colors.red,e,colors.black)

plot_lattices()
test_lattice_exception()



from utils import*
# from blochwave import bloch2D as bl          ;imp.reload(bl)
# from multislice import mupy_utils as mut     ;imp.reload(mut)
# import multislice.multi_2D as ms             ;imp.reload(ms)
plt.close('all')
opts = 'BM' #(Bloch) (Multislice)

K = 5.024937810560445   # exact Bragg condition for hk=(1,0) beam
keV = cst.lam2keV(1/K)

def setup():
    fig,axB=dsp.create_fig()
    cmap='jet'
    thick=1000  #thickness
    eps = 0.01  #strength of potential
    n = 10      #u=[1,n]
    e0=1
    # e0=0.98,0.965
    #### Bloch
    if 'B' in opts:
        file = 'dat/p1.txt'
        # p1 = mut.import_wallpp(file)
        # p1.plot_unit_cells(opts='uAa',nh=3,nk=3)
        b0  = bl.Bloch2D(file,keV=keV,u=[1,n],thick=thick,eps=e0*eps,
            Nmax=8,Smax=0.2,solve=1)
        # b0.show_beams(opts='B',fz=np.abs)
        # b0.show_ewald()
        # b0.show_beams(opts='S')
        b0.show_beams_vs_thickness(thicks=(0,thick,1000),strong=['I'],m={'I':100},
            linespec='--',marker='',cm=cmap,ax=axB,opt='')
        # b0.G('Sw')

        idx = b0.get_Istrong(Icols=['I'],out=1)
        ibs = b0.get_refl(idx)
        print('pendulosung thickness')
        print(b0.get_Xig())
setup()



from utils import*                  #;imp.reload(dsp)
import time
from EDutils import gui as vw   ;imp.reload(vw)
from utils import pytest_util    ;imp.reload(pytest_util)
from pynput.keyboard import Key, Controller
plt.close('all')
out,ref,dir=pytest_util.get_path(__file__)

# v=vw.Gui(cif_file='pets/alpha_glycine.cif',pets_opts='BKVr',xylims=1)
# v=vw.Gui(cif_file='diamond',path=out,pets_opts='Bkr',xylims=1,help=0)
v=vw.Gui(cif_file='diamond',path=out,config=out+'/config.pkl')#pets_opts='Bkr',xylims=1,help=0)

# tests = range(10,12)#[]
# tests = [0]
@pytest_util.add_link(__file__)
def test_viewer() :
    v=vw.Gui(cif_file='pets/alpha_glycine.cif',pets_opts='BKVr',xylims=1,
        pargs={'opt':''})

    return v.fig,v.ax

dt=0.5
def press(key):
    time.sleep(dt)
    print('pressing %s' %key)
    keyboard.press(  key)
    keyboard.release(key)

def test_keyboard():
    keyboard = Controller()
    from threading import Thread
    thread = Thread(target=test_q)
    thread.start()
    plt.show()

def test_q():
    press(key='3')
    press(key=' ')
    press(key='2')
    press(key='3')
    press(key='4')
    press(key='5')

    for k in v.keymap.df_keys.key.values:
        press(key='5')


# test_keyboard()

# fig,ax=test_viewer();fig.show()
# if -1 in tests:v = vw.Viewer()  #nothing should raise exception
# if 0 in tests:v = vw.Gui(path='dat/glycine',init_opts='',pets_opts='SKr')           #path alone (contains cif_file)
# if 1 in tests:v = vw.Viewer(path='dat/pets',init_opts='',rot=206)    #path alone (contains pts_file)
# if 2 in tests:v = vw.Viewer(cif_file='dat/glycine/alpha_glycine.cif')       #cif_file alone
# if 3 in tests:v = vw.Viewer(pets='dat/pets/glycine.pts',init_opts='i',F='Ig',rot=206)              #pets alone
# if 4 in tests:v = vw.Viewer(bloch='dat/bloch/Si110_200keV_bloch.pkl') #bloch alone
#
# if 5 in tests:v = vw.Viewer(path='dat/Si',cif_file='Si',u=[0,0,1])                  #path and cif
# if 6 in tests:v = vw.Viewer(path='dat/Si',bloch='dat/bloch/Si110_200keV_bloch.pkl') #path and bloch
# if 7 in tests:v = vw.Viewer(path='dat/glycine',pets='dat/pets/glycine.pts')         #path and pets
#
# #path and cif and bloch
# if 8 in tests:v = vw.Viewer(path='dat/Si',cif_file='Si',bloch='dat/bloch/Si110_200keV_bloch.pkl')
#
# #path and cif and pets
# if 9 in tests:v = vw.Viewer(path='dat/glycine',cif_file='dat/glycine/alpha_glycine.cif',pets='dat/pets/glycine.pts')
#
# #path and pets and bloch
# if 10 in tests:v = vw.Viewer(path='dat/glycine',
#     pets='dat/pets/glycine.pts',bloch='dat/glycine/alpha_glycine-103_200keV_bloch.pkl')
#
# #path and cif_file pets and bloch
# if 11 in tests:v = vw.Viewer(path='dat/glycine',cif_file='dat/glycine/alpha_glycine.cif',
#     pets='dat/pets/glycine.pts',bloch='dat/glycine/0001_bloch.pkl')



from utils import*
from scipy.signal import find_peaks
import importlib as imp
import multislice.nearBragg as nb   ;imp.reload(nb)
import wallpp.plane_group as pg     ;imp.reload(pg)
plt.close('all')
path=dsp.get_figpath(__file__,'/../../nearBragg/figures/')
dat='dat/'

keV=200
Natoms,dim = 3,2
ax,bz = 10,5        # lattice constants A
npx   = 4096       # number of pixels
Nx,Nz = 10,100,#10,2  # unit cells
qmax = 4
opts='F' #H(Holton)F(Fraunhofer)l d(diff)
eps=0.1

if 1:
    x,z,Za = [5],[2.5],[2]
    # pattern = np.array([[5,2.5,Za]])
    # x,z,Za = [0.25,0.75,0.5], [0.25,0.25,0.5], [1,1,2]
    # X   = np.random.rand(Natoms,dim)
    # Za  = np.random.randint(len(Zs),size=Natoms)
    np.save(dat+'atoms.npy',[x,z,Za])
pattern = np.load(dat+'atoms.npy')

# NB Greens
nbG = nb.NearBragg(pattern,ax,bz,keV=keV,npx=npx,Nx=Nx,Nz=Nz,qmax=qmax,method='Greens')
plts = [[nbG.getQ(),nbG.I,'g-+','Greens']]

if 'H' in opts:
    nbH = nb.NearBragg(pattern,ax,bz,keV=keV,npx=npx,Nx=Nx,Nz=Nz,qmax=qmax,method='Holton',path='dat/')
    objs=[(nbH,'k--+','Holton'),(nbG,'k-','Greens')]
    nb.display(objs,('q','I'),name=path+'comparisonHolton.svg',opt='sp')
    nbH.Pattern_show(name=path+'comparisonHoltonPattern.svg',opt='sp')

if 'F' in opts:
    nbFf = nb.NearBragg(pattern,ax,bz,keV=keV,eps=eps,npx=npx,Nx=Nx,Nz=Nz,qmax=qmax,method='Fraunhofer')
    # nbFr = nb.NearBragg(pattern,ax,bz,keV=keV,eps=eps,npx=npx,Nx=Nx,Nz=Nz,qmax=qmax,method='Fresnel')
    # objs=[(nbG,'k-','Greens'),(nbFr,'k--o','Fresnel'),(nbFf,'k--x','Fraunhofer')]
    # nb.display(objs,('q','I'))
    nbG.I/=nbG.I.max()
    # nbFr.I/=nbFr.I.max()
    nbFf.I/=nbFf.I.max()

    if 'd' in opts :
        tol = 5e-5
        # idx = nbG.I>1e-3
        idx = find_peaks(nbG.I,height=tol,prominence=tol,distance=10)[0]
        plts += [[nbG.getQ()[idx],nbG.I[idx],'gs','']]
    # plts+=[[nbFr.getQ(),nbFr.I,'r','Fresnel']]
    plts+=[[nbFf.getQ(),nbFf.I,'b','Fraunhofer']]
    dsp.stddisp(plts,labs=[r'$q(\AA^{-1})$','$I$'],xylims=[-2,2,0,1],
        name=path+'path_length.svg',opt='p')

    #plot differences
    if 'd' in opts:
        dI = 1e-15*np.ones(nbG.I.shape)
        dI[idx] = abs((nbG.I-nbFr.I)/nbG.I)[idx]
        lablog,xylims,xyTicks='',[-2,2,0,1],[0.5,0.1]
        if 'l' in opts:
            dI =np.log10(dI)
            lablog,xylims,xyTicks='\log_{10}',[-2,2,-10,5],[0.5,2]
        plts=[[nbFf.getQ(),dI,'r','']]
        labs = [r'$q(\AA^{-1})$','$%s |I_{fr}-I_{Gr}/I_Gr|$' %lablog]
        dsp.stddisp(plts,labs=labs,xylims=xylims,xyTicks=xyTicks,
            name=path+'path_length_diff.svg',opt='p')



from utils import*                  ;imp.reload(dsp)
from EDutils import utilities as ut ;imp.reload(ut)
from blochwave import bloch_pp as bl;imp.reload(bl)
from scipy.interpolate import interp1d

plt.close('all')

opts='t' #Solve(S) r(save rocking curve) t(tiff)
npts = 100
path='dat/bloch_new'

if 'S' in opts:
    bloch_args = {'cif_file':'dat/pets/alpha_glycine.cif','Smax':0.015,'Nmax':8,
        'solve':1,'thick':300,'keV':200}
    uvw = ut.get_uvw_rock(e0=[0,3,1],e1=[2,1],deg=2,npts=npts,show=0)
    rock = bl.Bloch_cont(path=path,tag='',uvw=uvw,Sargs=bloch_args)

rock_file=path+'/rock_.pkl'
rock = ut.load_pkl(file=rock_file)

if 't' in opts:
    rock.convert2tiff(thick=200,Imax=1e6,aperpixel=0.01,iz=-1)
    vw=rock.show_tiff(cutoff=20,frame=2)#,pargs={'opt':'p'},h=1)
    rock.convert2png(cutoff=20)

if 'r' in opts:
    # rock.set_beams_vs_thickness((10,200,3))
    # rock.plot_rocking(cond=lambda dfG:bl.strong_beams(dfG,tol=0.01,n=2),n=5)
    z,I = rock.get_rocking(cond=lambda dfG:bl.strong_beams(dfG,tol=0.01,n=2),n=5)
    for h,Ib in I.items() : I[h]=Ib[:,0]
    max([Ib.max() for Ib in I.values()])


    #frame info
    u = rock.df.u
    b = rock.load(10)
    Nmax=512//2
    px,py,I = b.df_G[['px','py','I']].values.T
    dq = 1.1*max(px.max(),py.max())/Nmax
    px,py = np.round(b.df_G.loc[hkl,['px','py']].values.T/dq)+Nmax
    dfpxy = pd.DataFrame(np.array([px,py]).T,columns=['px','py'],index=hkl)
    dfpxy.to_pickle(path+'pxy.pkl')


    #prepare interp dataframe
    hkl = list(I.keys())
    nbs,nframes = len(hkl),u.shape[0]
    Sw = {h:np.zeros((nframes)) for h in hkl}
    dfb = rock.beams.loc[list(I.keys())]
    for h,r in dfb.iterrows():
        Sw[h][r.Frame] = r.Sw
        Sw[h][:r.Frame[0]] = r.Sw[0]
        Sw[h][r.Frame[-1]:] = r.Sw[-1]
    # interp dataframe
    df = pd.DataFrame(columns=['Sw','I'],index=hkl)
    frame=np.arange(100)
    for h in hkl:
        df.loc[h,'Sw'] = interp1d(frame,Sw[h])
        df.loc[h,'I']= interp1d(frame,I[h])

    df.to_pickle(path+'/manim.pkl')

    df = pd.read_pickle(path+'/manim.pkl')
    # fb = [lambda t:(df.loc[h,'Sw'](t),df.loc[h,'I'](t)) for h in hkl]
    # for f in fb:print(f(0))
    fb=dict()
    for h in hkl:
        fb[h]=lambda t:(df.loc[h,'Sw'](t),df.loc[h,'I'](t))
        print(fb[h](0))



from utils import*
from EDutils import utilities as ut;imp.reload(ut)
from utils import pytest_util    ;imp.reload(pytest_util)
plt.close('all')

class C:
    def __init__(self,name,path,a,b=1,c=''):
        if not isinstance(c,str):
            raise TypeError('c must be of type str')
        self.a=a
        self.b=b
        self.c=c
        self.name=name
        self.path=path
        self.save()

    def _get_pkl(self):
        return ut.get_pkl(path=self.path,name=self.name)
    def save(self):
        print(self._get_pkl())
        ut.save(self,file=self._get_pkl())
    def __str__(self):
        return 'a=%d, b=%d, c=%s' %(self.a,self.b,self.c)

def test_sweep():

    df1 = ut.sweep_var(C,path='dat/sweep',params='a',vals=np.arange(2),tag='single')
    df2 = ut.sweep_var(C,path='dat/sweep',params=['a','b'],vals=np.random.rand(3,2),tag='double')
    df3 = ut.sweep_var(C,path='dat/sweep',params=['a','c'],vals=[[1,'test1'],[2,'test2']],tag='mixed')

    print('df1[0 ] : ',ut.load_pkl(df1.iloc[0].pkl))
    print('df2[1 ] : ',ut.load_pkl(df2.iloc[1].pkl))
    print('df3[-1] : ',ut.load_pkl(df3.iloc[-1].pkl))


@pytest_util.cmp_ref(__file__)
def test_get_uvw_cont():
    return ut.get_uvw_cont(u0=[1,0,1],u1=[1,1,1] ,nframes=100        )

@pytest_util.cmp_ref(__file__)
def test_get_uvw_rock():
    return ut.get_uvw_rock(e0=[1,0,1],e1=[1,0],npts=20,deg=1         )

@pytest_util.cmp_ref(__file__)
def test_get_uvw_CBED():
    return ut.get_uvw_CBED(u0=[1,1,1],cart=False ,npts=[10,20],deg=20)

@pytest_util.add_link(__file__)
def test_show_uvw():
    uvw=ut.get_uvw_cont(u0=[1,0,1],u1=[1,1,1] ,nframes=20)
    return ut.show_uvw(uvw,opt='',view=[50,-110])


@pytest_util.add_link(__file__)
def test_misc_orient():
    u   =ut.u_from_theta_phi(30,62)
    uvw =ut.get_uvw_from_theta_phi(30,64,nframes=6)
    fig,ax = ut.uvw_add_points(uvw,npts=1,plot=1,opt='')
    return fig,ax

@pytest_util.add_link(__file__)
def test_convert2tiff():
    import tifffile
    out,ref,dir=pytest_util.get_path(__file__)
    tiff_file = out+'convert.tiff'
    n=512
    x,y=np.meshgrid(range(-n,n),range(-n,n))
    im0=np.exp(-1e-3*x**2-1e-4*y**2)
    # dsp.stddisp(im=[im0],caxis=[0,1],pOpt='im')
    Imax=1e3
    ut.convert2tiff(tiff_file,im0,n0=n,rot=20,n=0.7*n,Imax=Imax)
    im=tifffile.imread(tiff_file)
    return dsp.stddisp(im=[im],caxis=[0,Imax],pOpt='im',opt='')



def test_import_crys():
    ut.import_crys('diamond')
    ut.import_crys('pets/alpha_glycine.cif')
    ut.import_crys()
    try:
        ut.import_crys('dummy')
    except Exception as e:
        print(colors.red,e,colors.black)


def test_remove_friedel_pairs():
    hkl = [[0,1,1],[0,1,-1],[0,-1,-1]]
    refl = [str(tuple(h)) for h in hkl]
    print(ut.remove_friedel_pairs(refl))



################################################################################
#### later
################################################################################
# def show_uvw():
#     fig,ax = dsp.create_fig(rc='3d',figsize='f')
#     ut.get_uvw_cont(u0=[1,0,1],u1=[1,1,1] ,nframes=100         )#title='cont'       ,**args)
#     ut.get_uvw_rock(e0=[1,0,1],e1=[0,1],npts=20,deg=1          )#title='rock_theta' ,**args)
#     ut.get_uvw_rock(e0=[1,0,1],e1=[1,0],npts=20,deg=1          )#title='rock_phi'   ,**args)
#     ut.get_uvw_CBED(u0=[1,1,1],cart=False ,npts=[10,20],deg=20 )#title='CBED'       ,**args)
#     plts = [[[0,u[0]],[0,u[1]],[0,u[2]],'b'] for u in uvw ]
#     return fig,ax

# test_remove_friedel_pairs()
# fig,ax=test_convert2tiff();fig.show()
# test_sweep()
# fig,ax=test_show_uvw();fig.show()
# fig,ax=test_misc_orient();fig.show()
# test_import_crys()



import importlib as imp
from utils import*
from crystals import Crystal
from scattering import scattering_factors as sf;imp.reload(sf)

q=np.linspace(0,2,1000)
fe = sf.get_fe(np.array([6]),q=q)

im,xylims,s='data/sf_kirkland.png',[0,2,0,20],q
im,xylims,s='data/sf_gruza.png',[0,1.5,0,6],q/2 #
fig = dsp.image_bg(im,labs=[r'$q=\sin\theta/\lambda (\AA^{-1})$','$f_e(\AA)$'],
    plots=[s,fe,'b-'],lw=2,xylims=xylims)
dsp.fix_pos(fig)



from utils import*
from scipy.signal import find_peaks
import importlib as imp
import wallpp.plane_group as pg   ;imp.reload(pg)
import multislice.nearBragg as nb ;imp.reload(nb)
import multislice.multi_2D as ms  ;imp.reload(ms)
plt.close('all')
path='../../multislice/docs_fig/multi2D/NBkinvsMS'
opts='I'

# Problem
keV   = 200        # wavelength 200keV
Nx    = 10         # Number of transverse unit cells
Nz    = 20          #
ndeg  = 2**8       # number of pixels
pptype,ax,bz,angle,Za = 'p1',5,10,90,  2
pattern = np.array([[2.5,5,Za]])
eps = 0.1
opt='p'

### Structure definition
p1 = pg.Wallpaper(pptype,ax,bz,angle,pattern,ndeg=ndeg)
potential = p1.get_potential_grid()


### Run multislice
ms0 = ms.Multi2D(potential,ax,bz,keV=keV,
    Nx=1,nz=Nz,dz=bz,eps=eps,
    iZs=1,iZv=1)
dq=ms0.q[1]-ms0.q[0]
# Ims = ms0.psi_qz[-1,:].copy()/(ms0.nx**2*ms0.dq**2)
# print('MS Inorm:%.2f' %(Ims.sum()*ms0.dq))
ms0.Bz_show(iBs=2,tol=1e-2,cmap='jet',lw=2)


### Run near bragg
npts=2049 #4097 #61
q0s = np.linspace(-3,3,npts)
nbG = nb.NearBragg(pattern.T,ax,bz,keV=keV,Nx=Nx,Nz=Nz,eps=eps,
    method='Greens2',q0s=q0s,iZv=1)
# nbG.F_show(qopt=0,lw=2)
# nbG.stat_show(lw=2,opt=opt,name=path+'_proba.svg')

#postprocessing
# Ncoh,Nkin = np.sum(nbG.S[0,:]),np.sum(nbG.S[1,:])
# dq = q0s[1]-q0s[0]
# Nscat = nbG.I.sum()*dq
# Ikin = nbG.I.copy()/Nscat*Nkin
# I = Ikin.copy()
# I[int(q0s.size/2)]+=Ncoh/dq
# print('Ncoh=%.4f, Nkin=%.4f,Nscat=%.4f, norm(I)=%.2f' %(Ncoh,Nkin,Nscat,np.sum(I)*dq))

#remove origin and normalize with highest peak
Ims100 = ms0.psi_qz[-1,:].copy()/ms0.nx**2*ms0.dq**2;
Ims100[0]=0
Ims100/=Ims100.max();


Inb100 = nbG.I.copy()
Inb100/=Inb100.max()
idx,props = find_peaks(Inb100, height=0.7, threshold=None, distance=10, prominence=0.75)#, width=None, wlen=None, rel_height=0.5, plateau_size=None)
hh = props['peak_heights'];hh[hh==1]=0
idx0 = idx[hh.argmax()]
Inb100/=Inb100[idx0]


pltsI=[]
# pltsI = [[q0s,Ikin,'g--','$I_{kin}$']]
pltsI+= [[q0s  ,Inb100 ,'g-','$NB$']]
pltsI+= [[ms0.q,Ims100 ,'bo','$MS$']]
dsp.stddisp(pltsI,labs=['$q(\AA^{-1})$','$I(q)$'],lw=2,
    name=path+'_I.svg',opt='p')



from utils import*                  #;imp.reload(dsp)
from blochwave import bloch_pp as bl;imp.reload(bl)
from blochwave import bloch         ;imp.reload(bloch)
from multislice import pets as pt   ;imp.reload(pt)
from EDutils import utilities as ut ;imp.reload(ut)
plt.close('all')
path = 'dat/bloch/optim/'

opts = 'F' #S(single), F(Full optim)
frame = 14

pets = pt.Pets('dat/pets/glycine.pts')
tag = 'glycine%s_' %str(frame).zfill(3)
bloch_args = {'cif_file':'dat/pets/alpha_glycine.cif',
    'keV':200,'Smax':0.025,'Nmax':9,'solve':1,'opts':'s'}

rpl = pets.rpl.loc[pets.rpl.F==frame]
rpl.index = rpl['hkl'].values.T
refl = list(rpl.index.values)

def get_cor(b,rpl,hkl):
    Irpl = np.array(rpl.loc[hkl,'I'],dtype=float)
    Iz = b.get_beams_vs_thickness(hkl,dict_opt=0)
    cor = np.abs([np.corrcoef(Irpl, I0)[0,1] for I0 in Iz.T])
    iZ  = np.argmax(cor)
    Iz0 = Iz[:,iZ]
    return hkl,cor,iZ,Iz0,Irpl

# single frame optimization
if 'S' in opts:
    rock = ut.load_rock(path,tag)

    b = bloch.Bloch(u=pets.uvw0[frame],path=path,name='test0',**bloch_args)
    # b.show_beams(refl=refl,opts='SVk',rot=203,xylims=2)
    hkl = b.get_beam(refl=refl,cond='',opt=0)
    # print(np.setdiff1d(refl,hkl))
    # b.set_beams_vs_thickness(thicks=(0,1000,1000));b.save()
    hkl,cor,iZ,Iz0,Irpl = get_cor(b,rpl,hkl)
    dsp.stddisp([Irpl,Iz0*Irpl.mean()/Iz0.mean(),'bo'],labs=['$I_o$','$I_c$'])

if 'F' in opts:
    if 0:
        u0 = pets.uvw0[frame]
        uvw = ut.get_uvw_CBED(u0=u0,deg=1,npts=5,plot=0,h3d=1)
        omega = np.arange(uvw.shape[0])
        # hkl = rock._get_refl(refl=refl,cond='')[0].tolist()
        rock = bl.bloch_rock(tag=tag,uvw=uvw,bloch_args=bloch_args,omega=omega,
            cond='',refl=refl,
            thicks=(0,500,100),
            W_args={'opts':'tf'},#'iTs':slice(2,14)},
            R_args={'opts':'f'},
            # S_args={'opts':'tf'},
            # ts0=0,
            # R_args={'cmap':'Spectral'},
            zs = np.arange(100,500,100),cm='hsv',hkls=[],
            path=path,opts='',opt='p')

    rock = ut.load_rock(path,tag)
    max_cor,max_cors = 0,np.zeros(rock.df.index.shape)
    for i,name in enumerate(rock.df.index):
        b = rock.load(i)
        hkl = b.get_beam(refl=refl,cond='',opt=0)
        if len(refl)-len(hkl)<10:
            print(i,name)
            hkl,cor,iZ,Iz,Irpl = get_cor(b,rpl,hkl)
            max_cors[i] = cor[iZ]
            if cor[iZ]>max_cor:
                max_cor = cor[iZ]
                hkl0,cor0,iZ0,Iz0,Irpl0,i0 = hkl,cor,iZ,Iz,Irpl,i

    # dsp.stddisp([Irpl0,Iz0*Irpl0.mean()/Iz0.mean(),'bo'],labs=['$I_o$','$I_c$'])



from utils import*                          ;imp.reload(dsp)
from EDutils import viewers as vw           ;imp.reload(vw)
from multislice import mupy_utils as mut    ;imp.reload(mut)
from multislice import pets as pt           ;imp.reload(pt)
import scipy.optimize as opt
plt.close('all')
pets = 'dat/pets/glycine.pts'

pets = pt.Pets('dat/pets/glycine.pts')
v0 = mut.Viewer(pets=pets,init='cr',rot=203,Nmax=13,Smax=0.025,frame=14)
# v = mut.Viewer(exp_path='dat/pets/tiff/processed',init='cr',rot=203,Nmax=13,Smax=0.025,frame=14)
# v = vw.Viewer(config='dat/config.pkl')

# v1 = vw.Viewer(path='dat',cif_file='dat/alpha_glycine.cif',init_opts='',
#     Nmax=6,Smax=0.025,tag='vw',dyn_opt=1,
#     pets_opts='SVkr',frame=17,thick=24,rotS=70,rot=205,multi=-1)
v = vw.Viewer(path='dat',cif_file='dat/alpha_glycine.cif',init_opts=' ',
    Nmax=9,Smax=0.025,tag='vw',dyn_opt=0,
    pets_opts='PBSkr',frame=14,thick=300,rot=203,multi=-1,rotS=70)



from utils import*
from blochwave import bloch as bl          ;imp.reload(bl)
from multislice import mupy_utils as mut     ;imp.reload(mut)
import multislice.multi_2D as ms             ;imp.reload(ms)
plt.close('all')


# file = 'dat/p1c.py'
thick=100

wallpp_args={'pp_type':'p1','a':1,'b':1,'angle':90,
    'pattern':np.array([[0.5,0.5,0]]),
    # 'alpha':90,'fract':True
    }

p1 = mut.import_wallpp(wallpp_args)
# p1.plot(opts='uAa',nh=3,nk=3)
p1.plot_unit_cells(opts='uAa',nh=3,nk=3)
# p1.show_structure_factor()

# b0  = bl.Bloch2D(wallpp_args,keV=200,u=[1,10],thick=thick,eps=0.5,
#     Nmax=10,Smax=0.1,solve=1)

# b0.show_ewald()
# b0.show_beams(opts='B',fz=np.abs)
# b0.show_beams(opts='SVB')
# b0.show_beams_vs_thickness(thicks=(0,thick,1000))
# b0.G('Sw')



from utils import*                  #;imp.reload(dsp)
from blochwave import bloch;imp.reload(bloch)
from blochwave import bloch_pp as bl;imp.reload(bl)
from multislice import pets as pt   ;imp.reload(pt)
from EDutils import utilities as ut #;imp.reload(ut)
# plt.close('all')
path = 'dat/bloch/'

frames = np.arange(10,30)
opts='P' #P(Pets) B(Bloch)c(convergence check)
opt='p'

pets = pt.Pets('dat/pets/glycine.pts')

if 'P' in opts:
    frames1 = np.setdiff1d(frames,np.arange(20,pets.nFrames+1,20))
    pets.show_Ihkl(name=path+'figures/glycine_Ihkl.svg',opt=opt)
    pets.show_Iavg(name=path+'figures/glycine_Iavg.svg',opt=opt)
    I = pets.show_Iframes(frames=frames1,refl=refl,cond='(I>500)',opts='AH',fz=fz,
        name=path+'figures/glycine_Iframe500.svg',lw=2,opt='ps')
    I = pets.show_Iframes(frames=frames1,refl=refl,
        Imax=500,cond='(I>100) & (I<500)',opts='AH',fz=fz,
        name=path+'figures/glycine_Iframe100.svg',lw=2,opt=opt)
    I = pets.show_Iframes(frames=frames1,refl=refl,
        cond='(I>25) & (I<100)',Imax=100,opts='AH',fz=fz,
        name=path+'figures/glycine_Iframe10.svg',lw=2,opt=opt)

    # I0 = pets.get_hklI(refl=I.columns.values)


bloch_args = {'cif_file':'dat/pets/alpha_glycine.cif','Smax':0.015,'Nmax':15,
    'opts':''}

npts = 4
alpha0,uvw0 = pets.alpha[frames-1],pets.uvw0[frames-1]
uvw = ut.uvw_add_points(uvw0,npts=npts,plot=0)
alpha = np.linspace(alpha0[0],alpha0[-1],uvw.shape[0])
# ut.show_uvw(uvw)

if 'c' in opts:
    b = bloch.Bloch(cif_file='dat/pets/alpha_glycine.cif',name='glycine14',u=uvw[14],keV=200,solve=0)
    b.convergence_test(Nmax=[7,15,30],Smax=[0.011,0.02],opts='Ss',
        z=np.arange(0,1001,1),hkl=[str(s) for s in [(0,0,2),(0,0,-2),(-3,-1,4)]])

alpha=alpha[10:20]
uvw=uvw[10:20]
if 'B' in opts:
    refl = [[-3,-1,2],[3,1,-4],[4,1,0],[-3,-1,4],[0,0,-2],[0,0,2]]
    # cond = '(Sw<1e-2) & (Vga>1e-6)'
    # cond='I>0.01'
    # cond = '(Sw<1e-2) & (Vga>1e-6) & (I>0.2)'
    fz = abs #np.log10
    cond=''
    # refl = [[0,0,-2],[0,0,2]] # [[-3,-1,2]]
    # refl = [[-4,0,0]]
    # refl = [[2,1,4]]

    hkls = [[hkl] for hkl in refl]
    # rock = bl.bloch_rock(tag='glycine_ref',uvw=uvw,bloch_args=bloch_args,
    #     omega=alpha,
    #     cond=cond,refl=refl,
    #     thicks=(10,1000,50),
    #     # W_args={'opts':'tf'},#'iTs':slice(2,14)},
    #     # R_args={'opts':'f'},
    #     # S_args={'opts':'tf'},
    #     # i=4,#ts0=None,
    #     # R_args={'cmap':'Spectral'},
    #     # zs = np.arange(100,500,100),cm='hsv',hkls=hkls,
    #     path=path,opts='',opt=opt)

    rock = ut.load_rock(path,tag='glycine_ref')
    f=rock.get_frames(str(tuple([0,0,2])),cols=['Sw'])
    # df  = pets.rpl
    # df0 = df.loc[(df.F<30) & (df.I>1)].sort_values('F')[['h','k','l','F']]
    # df0['refl'] = [str(h) for h in df0[['h','k','l']].values]
    # refl = pd.unique(df0.refl)
    # cond = ''#  '(Sw<1e-2) & I>1e-2'
    # Sw = rock.Sw_vs_theta(refl=refl,cond=cond,fz=fz,
    #     iTs=slice(2,30,1),opts='tfiI')

    # cond = 'I>0.01'
    # rock.Sw_vs_theta(refl=refl,cond='',fz=-np.log10,opts='tf');
    # rock.plot_rocking(zs=np.linspace(100,800,6),refl=[[-4,1,-1]],cond='',opts='t')
    # rock.plot_integrated(cond=cond,refl=refl,new=0,lw=2)



from utils import*
from wallpp import gui         ;imp.reload(gui)
from wallpp import config as cg;imp.reload(cg)
import sys,subprocess
plt.close('all')

# wp = gui.Wallpaper_gui(image='figures/cygne2_100.png',config=1,
#     path='dat',pg='p3')

def test_wiki_wallpp():
    for pp_type in cg.pp_types[1:]:
        image ='figures/%s.png' %pp_type
        wp = gui.Wallpaper_gui(image=image,path='dat',pg=pp_type,
            config=1,opt=0)

def convert_figures():
    files = sys.argv[1:]#'figures/p4g.png'
    df = cg.df_wallpp
    for f in files:
        pp_type = f.split('-')[1]                           #;print(pp_type)
        # pp_type = f.split('.png')[0]                           #;print(pp_type)
        if not pp_type in df.index:                         #
            pp_type = df.loc[df['pp_long'==pp_type]].name   #;print(pp_type)
        i = np.arange(17)[np.array(cg.pp_types)==pp_type][0]
        # old   = '%s.png' %pp_type                           #;print(image)
        image = 'wallpp_%s_%s.png' %(str(i).zfill(2),pp_type)    #;print(image)
        cmd = "convert %s -resize 400x400 %s" %(f,image)    #;print(cmd)
        # cmd = "mv %s %s" %(old,image)    #;print(cmd)
        subprocess.Popen(cmd,shell=True)

convert_figures()
def F(npts=21,lw=0.05,x0=0.5,y0=0.5):
    x,y = np.meshgrid(np.linspace(0,1,npts),np.linspace(0,1,npts))
    F = np.zeros((npts,npts))
    F[(y>0.15) & (y<0.75) & (abs(x-0.3)<=lw)]=1
    F[(abs(y-0.75)<=lw) & (x>0.3) & (x<0.7)]=1
    F[(abs(y-0.5)<=lw) & (x>0.3) & (x<0.55)]=1
    x,y = np.meshgrid(np.linspace(0,x0,npts),np.linspace(0,y0,npts))
    pattern = np.vstack([x.flatten(),y.flatten(),F.flatten()]).T
    np.save('dat/F.npy',pattern)



from utils import*                   ;imp.reload(dsp)
from EDutils import pets as pets_imp ;imp.reload(pets_imp)
from utils import pytest_util        ;imp.reload(pytest_util)
plt.close('all')
pts_path = 'pets/glycine.pts'
#make sure pets folder exists in dat

def test_import_pets():
    pets = pets_imp.Pets(pts_path,gen=True)#,lam=0.02508,aper=0.005340,omega=230,gen=1)

@pytest_util.add_link(__file__)
def test_show_exp():
    pets = pets_imp.Pets(pts_path)
    vw=pets.show_exp(frame=65,h=True,pargs={'opt':''})
    return vw.fig,vw.ax

# test_import_pets()
# test_show_exp()
# pets.show_uvw()
# pets.show_xyz()
# pets.show_hkl()


# pets.show_sim()
# pets.compare_xyz_pxpy(frame=frame,opts='oab')

# pets.show_xyz(view=[0,0],opt='sc',name='pets_glycine_xyz.png')
# pets.show_hkl(view=[0,0],opt='sc',name='pets_glycine_hkl.png')
# pets.show_frame(frame=frame,name='pattern_%d.png' %frame, opt='psc')
# pets.show_ewald_sphere(frame=19,Nmax=5,Smax=0.025,h3d=1)
# K = pets.get_beam(frame)
# df = mut.get_kinematic_pattern(cif_file,K,Nmax=5,Smax=0.02)
# pets.show_frames(frame=frame,thick=1000,Nmax=7,Smax=0.015,rot=206,Imag=10,opts='PKh',
#     v=0,pargs={'xylims':1.5})

# df = pets.compare_hkl(frame,Nmax=8,Smax=0.025,eps=None,v=0)

# frames = range(1,65) #[5,10,15,25,30]
# for f in frames:
#     pets.show_frame(frame=f,Smax=0.01,name='glycine_exp%d.png' %f,opt='sc')

# xylims = [-5,10,-5,10,-12,1]
# mut.show_cell(cif_file,title='%d' %frame,xylims=xylims,n=uvw,
#     axPos=[0,0,1,0.95],view=[0,0],h3D=1,opt='p')
# pets.mp4_crystal_rotation('glycine',xylims=xylims)



from utils import*                  ;imp.reload(dsp)
from EDutils import utilities as ut ;imp.reload(ut)
from blochwave import bloch_pp as bl;imp.reload(bl)
import tifffile
# plt.close('all')
path = 'dat/bloch/'

opt='p'
opts='Z' #RI' #S(Solve) s(single)
nframes = 200
deg  = 0.01

omega = np.arange(nframes)*deg
bloch_args = {'cif_file':'diamond','keV':200,
    'Nmax':8,'Smax':0.025,'solve':1,'opts':'sv'}

# uvw = ut.get_uvw(u0=[0,0,1],u1=[0,1,0],omega=omega)
# rock = bl.bloch_rock(tag='rx',uvw=uvw,
#     omega=omega,bloch_args=bloch_args,
#     thicks=(0,800,400),ts0=0,cond='(Sw<1e-3) &(Vga>1e-3)',
#     #zs = np.arange(168,800,168), hkls=[[[0,4,0]]],
#     path=path,opts='',opt='p')


# cond = '(Sw<2e-3) & (I>2e-2) & (Vga>1e-6)'
# cond = '(Sw<1e-4) & (I>1e-4) '
fz = lambda x : -np.log10(np.maximum(x,1e-10))
fz = lambda x:x
cond = ''#'(Sw<1e-2) & (Vga>1e-6)'
# hkls = [[-8,-2,2],[-5,1,1],[-1,7,-1],[3,7,-1],[-2,6,0]]
hkls = [[-8,-2,2],[-5,1,1],[-1,-7,1],[3,7,-1],[-2,6,0],[7,-3,-1],[5,1,-1]]

uvw = ut.get_uvw_from_theta_phi(theta=12,phi=19,omega=omega,plot=0)
# rock = bl.bloch_rock(tag='diamond_r1',uvw=uvw,
#     omega=omega,bloch_args=bloch_args,
#     thicks=(0,800,400),cond=cond,#thick=250,#fz=fz
#     # zs=np.arange(1,6)*50,hkls=hkls,ts0=2.0,
#     path=path,opts='',opt=opt)

rock = ut.load_pkl('dat/bloch/rock_diamond_r1.pkl')
# rock.integrate_rocking(cond=cond,refl=hkls,lw=2)
rock.Sw_vs_theta(cond='',refl=hkls,thick=250,fz=fz,opts='Iw')
# hkls = [[h] for h in hkls]


# rock.set_tag('diamond_r1')

# if 's' in opts:
#     if 'S' in opts:
#         bloch = bl.Bloch('diamond',u=uvw[0],keV=200,
#             Nmax=8,Smax=0.1,thick=thick,
#             name=frame(i0),path=path,solve=1,opts='sv')
#     else:
#         bloch = bl.load_Bloch(file=path+'%s.pkl' %frame(i0))
#     bloch.convert2tiff(path+'tiff/%s.tiff' %frame(i0),**tiff_args)
#     im = tifffile.imread(path+'tiff/%s.tiff' %frame(i0))
#     dsp.stddisp(im=[im],pOpt='im',caxis=[0,25],cmap='binary_r',bgcol='k')
#     # bloch.get_Istrong(m={'I':1e4})

    # from EDutils import viewers as vw       ;imp.reload(vw)
    # v = vw.Viewer(bloch=bloch)





def convert2tiff(thick,**kwargs):
    for i,b in enumerate(blochs):
        tiff_file=path+'tiff/'+frame(i)+'.tiff'
        b.set_thickness(thick)
        b.convert2tiff(tiff_file,**kwargs)

if 'c' in opts:
    pts_file = 'dat/pets/diamond.pts'
    aperpixel = 0.01
    tiff_args = {'Imax':7e4,'Nmax':256,'aperpixel':aperpixel,
        'gs3':0.04,'rmax':8e-5}

    convert2tiff(thick=thick,**tiff_args)
    lat_params = blochs[i0].crys.lattice_parameters
    ut.make_pets(pts_file,aperpixel,deg=deg,ref_cell=lat_params,tag=tag)



import importlib as imp
import httplib2, re
import pandas as pd,numpy as np,matplotlib.pyplot as plt
from utils import displayStandards as dsp;imp.reload(dsp)
plt.close('all')
df_file = 'dat/space_groups.pkl'

def get_space_groups():
    http = httplib2.Http()
    df = pd.DataFrame(columns=['space group','ops'])
    for i in range(1,231):
      headers, body = http.request("http://img.chem.ucl.ac.uk/sgp/large/%saz3.htm" %str(i).zfill(3))
      lines = body.decode().strip().split('\r\n')
      tle = [l for l in lines if '<TITLE>Space Group' in l][0]
      spg = tle.split(':')[1].split(';')[0]
      df.loc[i,'space group'] = spg

      print(spg)
      l1 = [i for i,l in enumerate(lines) if 'Symmetry operators' in l][0]
      l2 = [i for i,l in enumerate(lines) if '</PRE></FONT>' in l][0]
      ops = lines[l1+4:l2]
      df.loc[i,'ops'] = ops
      df.to_pickle(df_file)

# get_space_groups()

df = pd.read_pickle(df_file)
x,y,z = 0.1,0.2,0.7
dx={}
for i in df.index:
    c=df.loc[i]
    spg = c['space group']                              #;print(spg)
    ops = [l for l in c.ops if l.strip() != '']         #;print(ops)
    X = np.array([np.array(eval(op)) for op in ops])    #;print(X)
    X[X<0]+=1
    X[X>1]-=1
    dx[i] = X

    x0,y0,z0 = X.T
    c0 = dsp.getCs('Spectral',x0.size)
    fig,(ax0,ax1,ax2)=dsp.create_fig('f',rc=[1,3],pad=5)#;plt.subplots(1,3)
    dsp.stddisp(scat=[x0,y0,c0],ms=80,xylims=[0,1,0,1],xyTickLabs=[],pOpt='tXG',labs=['$x$','$y$'],ax=ax0,opt='')
    dsp.stddisp(scat=[y0,z0,c0],ms=80,xylims=[0,1,0,1],xyTickLabs=[],pOpt='tXG',labs=['$y$','$z$'],ax=ax2,opt='')
    dsp.stddisp(scat=[x0,z0,c0],ms=80,xylims=[0,1,0,1],xyTickLabs=[],pOpt='tXG',labs=['$x$','$z$'],ax=ax1,
        title='%d : $%s$' %(i,spg),name='figures/%s.png' %str(i).zfill(3),
        opt='sc')

    # dsp.stddisp(scat=[x0,y0,z0,'b'],ms=50,rc='3d',title='$%s$' %spg,
    #     xylims=1,#[0,1,0,1,0,1],
    #     name='figures/%s.png' %str(i).zfill(3),opt='sc')



from utils import*
import scipy.signal as sig
import importlib as imp
import wallpp.lattice as lat        ;imp.reload(lat)
import wallpp.plane_group as pg     ;imp.reload(pg)
import multislice.multi_2D as ms    ;imp.reload(ms)
plt.close('all')
path='../multislice/docs_fig/multi2D/'
opts='M'

K = 5.008343065817388   # exact Bragg condition for 2nd beam
keV   = cst.lam2keV(1/K)#200       # wavelength 200keV
Nx,Nz = 1,2000
ndeg  = 2**6       # number of pixels
npx   = Nx*ndeg    # number of pixels
pptype,ax,bz,angle = 'p1',2,2,90
Za = 2
pattern = np.array([[1,1,1]])
Nh = 20


#### 2-beam setup
rot = lambda t:np.array([[np.cos(t),np.sin(t)],[-np.sin(t),np.cos(t)]])
a1,a2 = lat.get_lattice_vec(lat_type='rect',a=ax,b=bz)
b1,b2 = lat.reciprocal_lattice_2D(a1,a2)
Ai = np.array([0.1,0.25,0.26,0.27,1.5])
fj = lambda ti,j:np.sqrt(np.pi*Ai[j])*np.exp(-(np.pi*Ai[j]*K*np.sin(ti))**2)


n = 10
x0,z0 = np.meshgrid(np.arange(-1,n),np.arange(n+1))
x0,z0 = np.stack([x0*ax+pattern[0][0],z0*bz+pattern[0][1]])
Xa = rot(1/n).dot(np.stack([x0.flatten(),z0.flatten()]))

ax1 = np.sqrt(1**2+n**2)*ax
bz1 = ax1

x,z   = np.meshgrid(np.arange(n*ndeg)*ax1/(n*ndeg),np.arange(n*ndeg)*bz1/(n*ndeg))
f = np.sum(np.array([pg.fv(np.stack([x.flatten(),z.flatten()]).T,X0,Za) for X0 in Xa.T]),axis=0)
f = np.reshape(f,(n*ndeg,n*ndeg))

if 'P' in opts:dsp.stddisp(im=[x,z,f],scat=[Xa[0],Xa[1],15,'k'],)

epss = np.arange(1,4)*0.01#array([0.01,0.05,0.1,0.2])

ms0 = []
for eps in epss:
    print(colors.red,eps,colors.black)
    ms0 += [ms.Multi2D([x[0],z.T[0],f],ax1,bz1,keV=keV,Nx=Nx,nz=Nz,
            dz=bz1/(2*n),eps=eps,sg=-1,
            iZs=1,iZv=np.inf)]

plts0,plts1 = [],[]
xi_0,xi_1   = np.zeros(epss.shape),np.zeros(epss.shape)
cs = dsp.getCs('hsv',epss.size)
for i,eps in enumerate(epss):
    Ib = ms0[i].Bz_show([10,20],out=1)
    idx0 = sig.find_peaks(-Ib[0],width=5)[0][0]
    idx1 = sig.find_peaks(-Ib[1],width=5,prominence=0.1)[0][0]
    xi_0[i] = ms0[i].z[idx0]
    xi_1[i] = ms0[i].z[idx1]
    plts0+=[[ms0[i].z,Ib[0],cs[i],r'$\epsilon=%.2f$' %eps]]
    plts1+=[[ms0[i].z,Ib[1],cs[i],r'$\epsilon=%.2f$' %eps]]
    plts0+=[[xi_0[i],Ib[0][idx0],[cs[i],'s'],'']]
    plts1+=[[xi_1[i],Ib[1][idx1],[cs[i],'s'],'']]
plts2 = [[epss,xi_0,'b-o','$g_1$'],[epss,xi_1,'g-o','$g_2$']]
dsp.stddisp(plts0 ,labs=[r'$z(\AA)$',r'$I$'],lw=2,name=path+'2_beam_I1.svg' ,opt='ps', xylims=[0,200,-0.0001,0.005])
dsp.stddisp(plts1 ,labs=[r'$z(\AA)$',r'$I$'],lw=2,name=path+'2_beam_I2.svg' ,opt='ps')
dsp.stddisp(plts2 ,labs=[r'Potential strength $V_{\epsilon}$',r'$\zeta(\AA)$'],lw=2,name=path+'2_beam_xi.svg' ,opt='ps')



import pandas as pd,numpy as np,crystals
from utils import*
from EDutils import pets as pt          ;imp.reload(pt)
from multislice import mupy_utils as mut   #;imp.reload(mut)
# I(integrate I over frames and compare with hkl.I)
# i(integrate on frame manually)
# c(Fo vs Fc_dyngo)
# o(check Fo=sqrt(Io) ok)
# h (show Ihkl)
# dyngo frames
path='figures/glycine_'
opts='h'
file = 'dat/pets/refinements/out.txt'
FoFc = 'dat/pets/refinements/FoFc.txt'
pets = pt.Pets('dat/pets/glycine.pts',gen=False,dyn=1)

pets_simu = pt.Pets('dat/pets_simu/glycine.pts',gen=False,dyn=1)
pets=pets_simu
#### sum/mean I over frames and compare with hkl.I
if 'I' in opts:
    # refl = str(tuple([0,0,2]))
    # I,F = pets.rpl.loc[pets.rpl.hkl==refl,['I','F']].values.T
    # dsp.stddisp([F,I,'b-',''],lw=2,labs=['F','I'])
    refl = pets.hkl.index
    sumI = [pets.rpl.loc[pets.rpl.hkl==r,'I'].sum() for r in refl]
    hklI = pets.hkl.I.values
    dsp.stddisp([sumI,hklI,'bo',''],labs=['sum(I)','hkl(I)'])


### frame integrate beams (manual fit) and compare to rpl.I
if 'i' in opts:
    df_Iint = pets.integrate_rpl(frames=np.arange(10,20))
    plts = [[df_Iint.Iint,df_Iint.Imax,'ro']]
    dsp.stddisp(plts,labs=['$I_{int}$','$I_{max}$'])
    plts = [[df_Iint.I,df_Iint.Im,'bo']]
    dsp.stddisp(plts,labs=['$I$','$Im$'])



## import dynamical refinement and keep unique reflections
df = pd.read_csv(FoFc,index_col=None,header=0,delimiter='  *',engine='python')
hkl = [str(tuple(h)) for h in df[['h','k','l']].values]
hkl,idx,c = np.unique(hkl,return_index=True,return_counts=True)
df = df.iloc[idx]
df.index = hkl


## Fo vs Fc from dyngo dynamical refinement
if 'c' in opts:
    plts=[]
    plts+=[[df.Fo,df.Fc,'bo']]
    dsp.stddisp(plts,labs=['$F_o$','$F_c$'],name=path+'FoFc.svg',opt='sc')
    print('Rfactor:%.1f%%' %(abs(np.sum(df.Fo**2)-np.sum(df.Fc**2))/sum(df.Fo**2)*100))



## checked that df.Fo=sqrt(hkl_dyn.Io)
if 'o' in opts or 'h' in opts:
    pets.HKL_dyn['hkl'] = [str(tuple(h)) for h in pets.HKL_dyn[['h','k','l']].values]
    idx = pets.HKL_dyn
    pets.HKL_dyn.index=pets.HKL_dyn['hkl']
    if 'o'  in opts:
        I = np.array([pets.HKL_dyn.loc[[h],'I'][0] for h in df.index])
        Fo  = np.sqrt(I)
        print('Fo-sqrt(I)=',(abs(df['Fo']-Fo)/Fo).max())
    if 'h' in opts:
        pets.show_Ihkl(name=path+'dyngo_beams.svg',opt='sc',dyn=df.index)
    # plts=[[df['Fo'],Fo,'rs']]
    # dsp.stddisp(plts,labs=['$F_o$','$F_c$'])




if 'd' in opts:
    v1=pets.show_sim()
    v2=pets.show_exp()




# if 0:
#     Zs = {'C':6,'O':8,'N':7,'H':1}
#     df = pd.read_csv(file,index_col=0,names=['x','y','z'],delimiter=' ')
#     df['Z'] = [c[0] for c in df.index]
#     df['Za'] = [Zs[Z] for Z in df.Z]
#
#     lat_vec,lat_params = pets.lat,pets.lat_params[:3]
#     pattern = np.hstack([df[['Za','x','y','z']].values,np.ones((df.shape[0],2))])
#     mut.make_xyz('dat/test.xyz',pattern,lat_vec,lat_params)
#     mut.show_grid3('dat/test.xyz',ms=50)
#
#     equiv= ['x,y,z',
#             '-x,-y,-z',
#             '-x+1/2,y+1/2,-z+1/2',
#             'x+1/2,-y+1/2,z+1/2',]
#     symmetry_operators = list(map(crystals.CIFParser.sym_ops_from_equiv, equiv))
#     atoms = [crystals.Atom(c.Z, coords = [c.x,c.y,c.z]) for n,c in df.iterrows()]
#
#     unitcell = crystals.crystal.symmetry_expansion(atoms, symmetry_operators)
#     glycine = crystals.Crystal(unitcell, lat_vec)



from utils import*
from math import factorial as fact
path='../../figures/dynamical_diffraction/Pcoh_kin_dual_dyn'
plt.close('all')

Nmax = 1000
N   = np.arange(Nmax)+1
sig = 0.01
ncol = 6

P = np.zeros((ncol,Nmax))
P[4] = sig**5*(N-3)*(N-4)*(N-5)*(N-6)*(N-7)/(5*4*3*2)/2
P[4] = sig**4*(N-2)*(N-3)*(N-4)*(N-5)/(4*3*2)/2
P[3] = sig**3*(N-1)*(N-2)*(N-3)/(3*2)/2
P[2] = sig**2*N*(N-1)/2 - P[3]
P[1] = sig*N-P[2]
P[0] = 1-P[1]

cs = dsp.getCs('jet',ncol)
plts=[[N,Pi,cs[i],'$P_%d$' %i] for i,Pi in enumerate(P)]
# plts+= [[N,P0,'b','P0']]
# plts+= [[N,P1,'r','P1']]
# plts+= [[N,P2,'g','P2']]
dsp.stddisp(plts,lw=2,labs=[r'$N$','P(z)'])

def scat(topt=3):
    rho   = 0.106   #atoms/A^3
    sig_e = np.array([0.001,0.0025,0.005]) #A^2
    le_s  = 1/(sig_e*rho) #A

    z = np.linspace(0,10000,1000) #A

    cs=dsp.getCs('jet',topt+2)
    l_e  = le_s[1]
    y = [1/fact(m)*(z/l_e)**m*np.exp(-z/l_e) for m in range(topt+1)]
    plts = [ [z/10,y[m],cs[m],'$%d$' %m] for m in range(topt+1)]
    plts+= [[z/10,1-np.array(y).sum(axis=0),'k--','']]
    dsp.stddisp(plts,lw=2,labs=[r'$z(nm)$','P(z)'],#title=tle,
        name=path+'%d.svg' %0, opt='p',fonts={'leg':20,'title':20})


def cross_section_sweep():
    for i,le in zip(range(le_s.size),le_s):
        Pcoh = np.exp(-z/le)
        Pkin = z/le*np.exp(-z/le)
        Pdual = 0.5*(z/le)**2*np.exp(-z/le)
        if topt:
            Ptri = 1/6*(z/le)**3*np.exp(-z/le)
            Pdyn = 1-np.exp(-z/le)*(1+z/le+0.5*(z/le)**2+(z/le)**3/6)
            S = Pcoh+Pkin+Pdual+Ptri+Pdyn
        else:
            Pdyn = 1-np.exp(-z/le)*(1+z/le+0.5*(z/le)**2)
            S = Pcoh+Pkin+Pdual+Pdyn

        tle = r'$\sigma_e=%.3f\AA^2$, $l_e$=%.d nm' %(sig_e[i],int(le/10))
        cs=dsp.getCs('jet',4)
        plts = [[z/10,Pcoh,cs[0],'$coh$'],[z/10,Pkin,cs[1],'$kin$']]
        plts+= [[z/10,Pdual,cs[2],'$double$']]
        if topt: plts+= [[z/10,Ptri ,cs[3],'$tri$']]
        plts+= [[z/10,Pdyn,'k','$dyn$']]#,[z/10,S,'k--']]
        dsp.stddisp(plts,lw=2,labs=[r'$z(nm)$','P(z)'],#title=tle,
            name=path+'%d.svg' %i, opt='p',fonts={'leg':20,'title':20})
# cross_section_sweep()
scat(topt=ncol-2)



from utils import*
from blochwave import bloch2D as bl          ;imp.reload(bl)
from multislice import mupy_utils as mut     ;imp.reload(mut)
import multislice.multi_2D as ms             ;imp.reload(ms)
plt.close('all')
opts = 'BM' #(Bloch) (Multislice)

K = 5.024937810560445   # exact Bragg condition for hk=(1,0) beam
keV = cst.lam2keV(1/K)

fig,axB=dsp.create_fig()
cmap='jet'
thick=1000  #thickness
eps = 0.01  #strength of potential
n = 10      #u=[1,n]
e0=1
# e0=0.98,0.965
#### Bloch
if 'B' in opts:
    file = 'dat/p1.py'
    # p1 = mut.import_wallpp(file)
    # p1.plot_unit_cells(opts='uAa',nh=3,nk=3)
    b0  = bl.Bloch2D(file,keV=keV,u=[1,n],thick=thick,eps=e0*eps,
        Nmax=8,Smax=0.2,solve=1)
    # b0.show_beams(opts='B',fz=np.abs)
    # b0.show_ewald()
    # b0.show_beams(opts='S')
    b0.show_beams_vs_thickness(thicks=(0,thick,1000),strong=['I'],m={'I':100},
        linespec='--',marker='',cm=cmap,ax=axB,opt='')
    # b0.G('Sw')

    idx = b0.get_Istrong(Icols=['I'],out=1)
    ibs = b0.get_refl(idx)
    print('pendulosung thickness')
    print(b0.get_Xig())




#multislice
if 'M' in opts:
    file_ms = 'dat/2_beam.npy'
    ax,bz = b0.crys.params[:2]
    pattern = b0.crys.pattern
    ax1 = np.sqrt(1**2+n**2)
    bz1 = ax1

    if 'V' in opts:
        import wallpp.plane_group as pg     ;imp.reload(pg)
        x1,y1,Za = pattern[0]
        x0,z0 = np.meshgrid(np.arange(-2,n+1),np.arange(-1,n+2))
        x0,z0 = np.stack([x0*ax+x1,z0*bz+y1])
        rot = lambda t:np.array([[np.cos(t),np.sin(t)],[-np.sin(t),np.cos(t)]])
        Xa = rot(np.arctan(1/n)).dot(np.stack([x0.flatten(),z0.flatten()]))

        ndeg=2**6
        x,z = np.meshgrid(np.arange(n*ndeg)*ax1/(n*ndeg),np.arange(n*ndeg)*bz1/(n*ndeg))
        f = np.sum(np.array([pg.fv(np.stack([x.flatten(),z.flatten()]).T,X0,int(Za)) for X0 in Xa.T]),axis=0)
        f = np.reshape(f,(n*ndeg,n*ndeg))
        np.save(file_ms,[x,z,f])
        print(colors.yellow+file_ms+colors.green+" saved"+colors.black)
        if 'v' in opts:scat=[Xa[0],Xa[1],15,'k']

    x,z,f = np.load(file_ms,allow_pickle=True)
    dz = bz1/(n)
    Nz = np.int(thick/dz) #1000*n
    tilt = np.linspace(0,0.08,40)[28]

    ms0 = ms.Multi2D([x[0],z.T[0],f],ax1,bz1,keV=keV,Nx=1,nz=Nz,
            dz=dz,eps=eps,sg=-1,
            iZs=1,iZv=np.inf,tilt=tilt)
    # ms0.V_show()
    # ms0.Qz_show([-1],opts='nO',xylims=['x',-6,6],lw=2)
    ms0.Bz_show(np.arange(0,2,1)*n,lw=2,ax=axB,opt='',cmap=cmap)

fig.show()



from utils import*                      #;imp.reload(dsp)
from blochwave import bloch             ;imp.reload(bloch)
from EDutils import utilities as ut     #;imp.reload(ut)
from multislice import mupy_utils as mut#;imp.reload(mut)
plt.close('all')
cif = 'dat/bloch/Si3N4/Si3N4.cif'
uza = [0,0,1]

crys = mut.import_crys(cif)
lat_vec = np.array(crys.lattice_vectors)
u = lat_vec.T.dot(np.array(uza))
b = bloch.Bloch(cif_file=cif,keV=200,u=u,Smax=0.01,Nmax=5,solve=0,opts='s')



from bs4 import BeautifulSoup
import os,sys,argparse #.path import path
from subprocess import check_output,Popen,PIPE
from colorama import Fore as colors

usage='''Usage:
report.py [opt] [libs]

OPTIONS
-------
    opt  : h(help) s(skip running tests) l(libs) f(file serv) p(python)
    libs : <lib1>,<lib2>,.. if 'l' in opts
    python : python executable for subprocess
'''


def get_report_summary(file):
    body=open(file,'rb')
    soup = BeautifulSoup(body,'html.parser')
    tests =  '|'.join([ soup.find_all('span',s)[0].get_text().split(' ')[0]
        for s in ['passed','skipped','failed','error']])
    # link_name = os.path.basename(file)#'report'
    summary= "[%s](%s) | %s " %('tests_report',reports_lnk(file),tests)

    return summary

def get_coverage_summary(file):
    body=open(file,'rb')
    soup = BeautifulSoup(body,'html.parser')
    # cov=soup.find_all('tfoot')[0].find('td','right').get_text()
    cov=soup.find_all('span','pc_cov')[0].get_text()
    return " [%s](%s) | %s " %('cov_report',reports_lnk(file),cov)

def run_tests():

    report="# Report\n\n"
    report+="lib | report | passed | skipped | failed | errors | cov_report | coverage \n"
    report+="--  | --     | --     | --      | --     | --     | --         | --       \n"

    for lib in libs:
        report_file = '%s/%s/report/report.html'  %(tests_dir,lib)
        cov_file    = '%s/%s/htmlcov/index.html'  %(tests_dir,lib)

        #run test
        args="--html=%s --self-contained-html --cov-report=html --cov=../../%s" %(report_file,lib)
        cmd ='%s -m pytest %s' %(python,args)
        job ='cd %s/%s; if [ ! -d report ];then mkdir report;fi; %s ' %(tests_dir,lib,cmd)
        print(cmd)
        if run_opt:
            # oe=check_output(job, shell=1).decode().strip()
            p=Popen(job, shell=1);p.wait();o,e=p.communicate()
            if o  or e:print(o,e)

        #get summary
        lib_report=get_report_summary(report_file)
        lib_cov   =get_coverage_summary(cov_file)

        report+="%s | %s | %s \n" %(lib,lib_report,lib_cov)

    report_file=os.path.join(tests_dir,"report.md")
    with open(report_file,"w") as f: f.write(report)

    #convert to html
    html_file=report_file.replace('.md','.html')
    oe=check_output("pandoc %s > %s" %(report_file,html_file),shell=True).decode()
    print(colors.YELLOW+reports_lnk(html_file)+colors.RESET)


if __name__=="__main__":
    opt = ''

    tests_dir = os.path.realpath(os.path.dirname(__file__))#;print(tests_dir)
    hostname    = check_output('hostname -A', shell=1).decode().strip().split()[-1]
    reports_lnk = lambda file:"http://%s:8010/%s" %(hostname,file.split('tests/')[1])
    # libs = ['blochwave','EDutils']#,'multislice']
    # if len(sys.argv)>1:
    #     opt = sys.argv[1]
    #     if 'h' in opt:
    #         print(usage )
    #         sys.exit()
    #     if 'f' in opt:
    #         reports_lnk=lambda file:"file://"+os.path.realpath(file)
    #     if 's' in opt:
    #         run_opt=False
    #     if 'l' in opt:
    #         libs=sys.argv[2].split(',')
    #     if 'p' in opt:
    #         python = sys.argv[3]


    parser = argparse.ArgumentParser()
    parser.add_argument('-f','--file'  ,action='store_true')
    parser.add_argument('-s','--skip'  ,action='store_true')
    parser.add_argument('-l','--libs'  ,default='blochwave,EDutils')
    parser.add_argument('-p','--python',default='python3')

    args = parser.parse_args()
    if args.file:reports_lnk=lambda file:"file://"+os.path.realpath(file)
    run_opt=not args.skip
    libs  = [ s.replace('/','') for s in args.libs.split(',')]
    python=args.python

    print(run_opt,libs,python)
    run_tests()
    # write_report()



from utils import*
from matplotlib import patches
from wallpp import config as cg      ;imp.reload(cg)



for pp_type in cg.pp_types:
    verts = cg.df_wallpp.loc[pp_type,'asym_cell']
    pp = patches.Polygon(verts,alpha=0.3,edgecolor='k',linewidth=1.5)
    dsp.stddisp(patches=[pp],title=pp_type)



from utils import*
import gemmi

crambin = gemmi.read_pdb('data/1ejg.pdb')
residues = crambin[0][0]
atoms =[]
for res in residues:
    atoms += [[a.element.atomic_number] + a.pos.tolist() for a in res]



from utils import*
import cbf,tifffile, glob,os

exp = '../simus/ireloh2/exp/'
out = 'ireloh2/'


def convert_files(exp,out):
    cbf_files = glob.glob(exp+'*.cbf')
    print(colors.green+'file saved'+colors.black)
    for file in cbf_files :
        try:
            data = cbf.read(file).data
            base_file = os.path.basename(file).replace('.cbf','')
            tif_file = out+'tiff/'+base_file+'.tiff'
            tifffile.imwrite(tif_file,data)
            print(colors.yellow+tif_file+colors.black)
        except:
            print('error with file %s' %file)
def make_pets(pts_file,aperpixel,deg=0.0652,ref_cell=None):
    center,pixel_size='AUTO',0.01,
    phi,omega= deg/2 ,0
    ax,by,cz,alpha,beta,gamma = ref_cell
    # ax,by,cz,alpha,beta,gamma = 8.1218, 9.977, 17.725, 90.0, 90.0, 90.0
    pts = '''lambda 0.025080
geometry continuous
omega  %d
phi  %.5f
virtualframes   7 5 1

aperpixel    %.6f
noiseparameters      2.5000      1.0000
saturationlimit   20000

center    %s
beamstop no

dstarmax  1.800
dstarmaxps  2.0
i/sigma    7.00    5.00
reflectionsize  10

referencecell     %.5f    %.5f     %.5f    %.5f   %.5f    %.5f 1

#List of images
#columns: file name,alpha,beta,delta omega,frame scale,calibration correction(px/rec.angstrom),ellipt.distortion correction(amplitude),ellipt.distortion correction(phase), use for calculations(0/1)
imagelist
'''%(omega,phi,aperpixel,center,  ax,by,cz,alpha,beta,gamma)
    tif_files = np.sort(glob.glob(out+'/tiff/*.tiff'))
    alphas = np.arange(tif_files.size)*deg
    for i,tif_file in enumerate(tif_files):
        tif_file = os.path.basename(tif_file)
        pts+='%s %.4f 0.0 0.0 1.0 0 0 0  1\n' %('tiff\\'+tif_file,alphas[i])
    pts+='endimagelist\n'
    # pts_file = out+name+'.pts'
    with open(pts_file,'w') as f:
        f.write(pts)
        print(colors.green+'file saved : '+colors.yellow+pts_file+colors.black)

# convert_files(exp,out)
make_pets('ireloh')
